"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLatestBlock = exports.registerCache = void 0;
const config_1 = require("../config");
const block_1 = require("../utils/block");
const db = __importStar(require("../storage/sqlite3storage"));
const utils_1 = require("../utils");
let latestBlock = null;
let lastCacheUpdateTimestamp = 0;
let getLatestBlockFromDB_running = false;
async function getLatestBlockFromDB() {
    const before = Date.now();
    const delayInMillis = (0, block_1.blockQueryDelayInMillis)();
    const sql = `SELECT * FROM (SELECT * FROM blocks ORDER BY number DESC LIMIT 100) AS subquery WHERE timestamp <= ${Date.now() - delayInMillis}`;
    //Shouldn't the highest block number always be the latest block?
    //This query seems much faster for the same thing
    // A:  nope, some reason this only updates once only 10 cycles or so.. even if they come to us out 
    // of order I would think the sort on primary key would still work... oh... it could be because
    // earlier blocks are sent.   I think maybe this also needs the timestamp check,
    // or we get 10 and then sort through and find one that works 
    //const sql = `SELECT * FROM blocks ORDER BY number DESC LIMIT 1`
    const block = await db.get(sql);
    const elapsed = Date.now() - before;
    console.log('SLOW QUERY?  getLatestBlockFromDB   ', elapsed, 'ms', block.number);
    return block;
}
async function updateLatestBlockCacheIfNeeded() {
    const now = Date.now();
    if (latestBlock === null || now - lastCacheUpdateTimestamp >= config_1.config.blockCache.cacheUpdateIntervalInMillis) {
        try {
            if (getLatestBlockFromDB_running === true) {
                while (getLatestBlockFromDB_running === true) {
                    await (0, utils_1.sleep)(200);
                }
                console.log('SLOW QUERY?  getLatestBlockFromDB skipped', latestBlock.number);
                return; //other process updated latestBlock
            }
            getLatestBlockFromDB_running = true;
            latestBlock = await getLatestBlockFromDB();
            lastCacheUpdateTimestamp = now;
            getLatestBlockFromDB_running = false;
        }
        finally {
            getLatestBlockFromDB_running = false;
        }
    }
}
function registerCache() {
    if (config_1.config.blockCache.enabled) {
        setInterval(updateLatestBlockCacheIfNeeded, config_1.config.blockCache.cacheUpdateIntervalInMillis);
    }
}
exports.registerCache = registerCache;
async function getLatestBlock() {
    if (config_1.config.blockCache.enabled) {
        await updateLatestBlockCacheIfNeeded();
        return latestBlock;
    }
    const block = await getLatestBlockFromDB();
    return block;
}
exports.getLatestBlock = getLatestBlock;
//# sourceMappingURL=LatestBlockCache.js.map