"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getContractInfo = exports.decodeTx = exports.ZERO_ETH_ADDRESS = exports.ZERO_ADDRESS = void 0;
const types_1 = require("../types");
const transaction_1 = require("../storage/transaction");
const web3_1 = __importDefault(require("web3"));
const account_1 = require("../storage/account");
const log_1 = require("../storage/log");
const index_1 = require("../config/index");
const ERC20_json_1 = __importDefault(require("../utils/abis/ERC20.json"));
const ERC721_json_1 = __importDefault(require("../utils/abis/ERC721.json"));
const ERC1155_json_1 = __importDefault(require("../utils/abis/ERC1155.json"));
const util_1 = require("@ethereumjs/util");
const rlp_1 = require("@ethereumjs/rlp");
const ERC_721_INTERFACE = '0x80ac58cd';
const ERC_1155_INTERFACE = '0xd9b67a26';
const ERC_TOKEN_METHOD_DIC = {
    '0xa9059cbb': 'Transfer',
    '0xa978501e': 'Transfer From',
    '0x23b872dd': 'Transfer From',
    '0x095ea7b3': 'Approve',
    '0xe63d38ed': 'Disperse ETH',
    '0xc73a2d60': 'Disperse Token',
    '0xf242432a': 'Safe Transfer From',
    '0x42842e0e': 'Safe Transfer From',
    '0x2eb2c2d6': 'Safe Batch Transfer From',
    '0xa22cb465': 'Set Approval For All',
    '0x3593cebc': 'Batch Transfer',
    '0x4885b254': 'Batch Transfer From',
    '0xc9c65396': 'Create Pair',
    '0x02751cec': 'Remove Liquidity ETH',
    '0x1249c58b': 'Mint',
    '0x7ff36ab5': 'Swap Exact SHM For Tokens',
    '0x18cbafe5': 'Swap Exact Tokens For SHM',
    '0x38ed1739': 'Swap Exact Tokens For Tokens',
    '0x2e1a7d4d': 'Withdraw',
    '0xd0e30db0': 'Deposit',
    '0xf305d719': 'Add Liquidity ETH',
    '0xe8e33700': 'Add Liquidity',
};
const ERC_TOKEN_APPROVAL_EVENT = '0x8c5be1e5ebec7d5bd14f71427d1e84f3dd0314c0f7b2291e5b200ac8c7c3b925'; // ERC20, ERC721
const ERC_TOKEN_TRANSFER_EVENT = '0xddf252ad1be2c89b69c2b068fc378daa952ba7f163c4a11628f55a4df523b3ef'; // ERC20, ERC721
const ERC_TOKEN_APPROVAL_FOR_ALL_EVENT = '0x17307eab39ab6107e8899845ad3d59bd9653f200f220920489ca2b5937696c31'; // ERC721, ERC1155
const ERC_TOKEN_TRANSFER_SINGLE_EVENT = '0xc3d58168c5ae7397731d063d5bbf3d657854427343f4c083240f7aacaa2d0f62'; // ERC1155
const ERC_TOKEN_TRANSFER_BATCH_EVENT = '0x4a39dc06d4c0dbc64b70af90fd698a233a518aa5d07e595d983b8c0526c8f7fb'; // ERC1155
exports.ZERO_ADDRESS = '0x0000000000000000000000000000000000000000000000000000000000000000';
exports.ZERO_ETH_ADDRESS = '0x0000000000000000000000000000000000000000';
const UNISWAP_SWAP_EVENT = '0xd78ad95fa46c994b6551d0da85fc275fe613ce37657fb8d5e3d130840159d822';
const UNISWAP_SYNC_EVENT = '0x1c411e9a96e071241c2f21f7726b17ae89e3cab4c78be50e062b03a9fffbbad1';
const UNISWAP_DEPOSIT_EVENT = '0xe1fffcc4923d04b559f4d29a8bfc6cda04eb5b0d3c460751c2402c5c5cc9109c';
const UNISWAP_WITHDRAWAL_EVENT = '0x7fcf532c15f0a6db0bd6d0e038bea71d30d808c7d98cb3bf7268a95bf5081b65';
// const UNISWAP_MINT_EVENT = '0x4c209b5fc8ad50758f13e2e1088ba56a560dff690a1c6fef26394f4c03821c4f'
// const UNISWAP_BURN_EVENT = '0xdccd412f0b1252819cb1fd330b93224ca42612892bb3f4f789976e6d81936496'
const ERC_20_BALANCE_SLOT = '0x0';
const ERC_721_BALANCE_SLOT = '0x3';
const ERC_1155_BALANCE_SLOT = '0x3'; // This is not correct; have to research and update it later
const decodeTx = async (tx, storageKeyValueMap = {}, newTx = true) => {
    var _a, _b, _c, _d, _e, _f;
    const txs = [];
    const accs = [];
    const tokens = [];
    if ('readableReceipt' in tx.wrappedEVMAccount && ((_a = tx.wrappedEVMAccount.readableReceipt) === null || _a === void 0 ? void 0 : _a.status) === 0) {
        if (((_b = tx.wrappedEVMAccount.readableReceipt) === null || _b === void 0 ? void 0 : _b.logs) && ((_c = tx.wrappedEVMAccount.readableReceipt) === null || _c === void 0 ? void 0 : _c.logs.length) > 0) {
            console.log('Failed EVM Transaction but with logs', tx.txHash);
            console.dir(tx, { depth: null });
        }
    }
    const logs = 'readableReceipt' in tx.wrappedEVMAccount && ((_d = tx.wrappedEVMAccount.readableReceipt) === null || _d === void 0 ? void 0 : _d.logs);
    if (logs === null || logs === undefined)
        return { txs, accs, tokens };
    if (!newTx && index_1.config.processData.decodeTokenTransfer) {
        // Check if there is any tokenTx with this txId; if found, return empty result to skip decoding
        const tokenTxs = await (0, transaction_1.queryTokenTxByTxId)(tx.txId);
        if (tokenTxs.length > 0) {
            return { txs, accs, tokens };
        }
    }
    if (logs && logs.length > 0) {
        let TransferTX = false;
        for (const log of logs) {
            const logToSave = {
                cycle: tx.cycle,
                timestamp: tx.timestamp,
                txHash: tx.txHash,
                blockNumber: parseInt(log.blockNumber),
                blockHash: log.blockHash,
                contractAddress: log.address,
                log: log,
                topic0: '',
            };
            log.topics.forEach((topic, j) => {
                logToSave[`topic${j}`] = topic;
            });
            (0, log_1.insertLog)(logToSave);
            if (!index_1.config.processData.decodeTokenTransfer)
                continue;
            let tokenTx = null;
            if (log.topics) {
                if (log.topics.includes(ERC_TOKEN_TRANSFER_EVENT)) {
                    if (!TransferTX && txs.length > 0) {
                        if (txs[0].tokenEvent === 'Approval')
                            txs.pop();
                    }
                    TransferTX = true;
                    let tokenEvent = 'Transfer';
                    if (log.topics[1] === exports.ZERO_ADDRESS)
                        tokenEvent = 'Mint';
                    else if (log.topics[2] === exports.ZERO_ADDRESS)
                        tokenEvent = 'Burn';
                    tokenTx = {
                        tokenType: log.topics[3] ? types_1.TransactionType.ERC_721 : types_1.TransactionType.ERC_20,
                        tokenFrom: `0x${log.topics[1].substring(26)}`.toLowerCase(),
                        tokenTo: `0x${log.topics[2].substring(26)}`.toLowerCase(),
                        tokenValue: log.topics[3] || log.data,
                        tokenEvent,
                    };
                }
                else if (log.topics.includes(ERC_TOKEN_APPROVAL_FOR_ALL_EVENT)) {
                    const accountExist = await (0, account_1.queryAccountByAccountId)(log.address.slice(2).toLowerCase() + '0'.repeat(24) //Search by Shardus address
                    );
                    let tokenType = types_1.TransactionType.ERC_721;
                    if (accountExist && accountExist.contractType === types_1.ContractType.ERC_1155)
                        tokenType = types_1.TransactionType.ERC_1155;
                    tokenTx = {
                        tokenType,
                        tokenFrom: `0x${log.topics[1].substring(26)}`.toLowerCase(),
                        tokenTo: `0x${log.topics[2].substring(26)}`.toLowerCase(),
                        tokenValue: log.data,
                        tokenEvent: 'Approval For All',
                    };
                    if (tokenType === types_1.TransactionType.ERC_1155)
                        tokenTx.tokenOperator = null;
                }
                else if (log.topics.includes(ERC_TOKEN_TRANSFER_SINGLE_EVENT)) {
                    if (!TransferTX && txs.length > 0) {
                        if (txs[0].tokenEvent === 'Approval')
                            txs.pop();
                    }
                    TransferTX = true;
                    let tokenEvent = 'Transfer Single';
                    if (log.topics[2] === exports.ZERO_ADDRESS)
                        tokenEvent = 'Mint';
                    else if (log.topics[3] === exports.ZERO_ADDRESS)
                        tokenEvent = 'Burn';
                    if (index_1.config.verbose)
                        if (log.data.length == 130) {
                            console.log(log.data.substring(0, 66));
                            console.log(web3_1.default.utils.hexToNumberString(`${log.data.substring(0, 66)}`));
                            console.log(log.data.substring(66, 130));
                            console.log(web3_1.default.utils.hexToNumberString(`0x${log.data.substring(66, 130)}`));
                        }
                    tokenTx = {
                        tokenType: types_1.TransactionType.ERC_1155,
                        tokenFrom: `0x${log.topics[2].substring(26)}`.toLowerCase(),
                        tokenTo: `0x${log.topics[3].substring(26)}`.toLowerCase(),
                        tokenValue: log.data,
                        tokenEvent,
                        tokenOperator: `0x${log.topics[1].substring(26)}`.toLowerCase(),
                    };
                }
                else if (log.topics.includes(ERC_TOKEN_TRANSFER_BATCH_EVENT)) {
                    if (!TransferTX && txs.length > 0) {
                        if (txs[0].tokenEvent === 'Approval')
                            txs.pop();
                    }
                    TransferTX = true;
                    let tokenEvent = 'Transfer Batch';
                    if (log.topics[2] === exports.ZERO_ADDRESS)
                        tokenEvent = 'Mint';
                    else if (log.topics[3] === exports.ZERO_ADDRESS)
                        tokenEvent = 'Burn';
                    try {
                        const web3 = new web3_1.default();
                        const result = web3.eth.abi.decodeParameters(['uint256[]', 'uint256[]'], log.data);
                        if (index_1.config.verbose)
                            console.log('Transfer Batch Decoding', result);
                        if ((result === null || result === void 0 ? void 0 : result['0']) &&
                            result['1'] &&
                            result['0'].length === result['1'].length) {
                            for (let i = 0; i < result['0'].length; i++) {
                                // Created a specail technique to extract the repective tokenValue of each tokenId/value transfer
                                /* eslint-disable security/detect-object-injection */
                                const id = `${web3_1.default.utils.padLeft(web3_1.default.utils.numberToHex(result['0'][i]), 64)}`;
                                const value = `${web3_1.default.utils.padLeft(web3_1.default.utils.numberToHex(result['1'][i]), 64)}`.slice(2);
                                /* eslint-enable security/detect-object-injection */
                                const tokenValue = id + value;
                                tokenTx = {
                                    tokenType: types_1.TransactionType.ERC_1155,
                                    tokenFrom: `0x${log.topics[2].substring(26)}`.toLowerCase(),
                                    tokenTo: `0x${log.topics[3].substring(26)}`.toLowerCase(),
                                    tokenValue,
                                    tokenEvent: 'Transfer Batch',
                                    tokenOperator: `0x${log.topics[1].substring(26)}`.toLowerCase(),
                                };
                                txs.push({
                                    ...tokenTx,
                                    contractAddress: log.address,
                                });
                                if (tokenTx.tokenTo !== tx.txFrom &&
                                    tokenTx.tokenTo !== tx.txTo &&
                                    !accs.includes(tokenTx.tokenTo)) {
                                    accs.push(tokenTx.tokenTo.toLowerCase());
                                }
                            }
                            continue;
                        }
                    }
                    catch (e) {
                        console.log('Error in decoding transferBatch', e);
                        tokenTx = {
                            tokenType: types_1.TransactionType.ERC_1155,
                            tokenFrom: `0x${log.topics[2].substring(26)}`.toLowerCase(),
                            tokenTo: `0x${log.topics[3].substring(26)}`.toLowerCase(),
                            tokenValue: log.data,
                            tokenEvent,
                            tokenOperator: `0x${log.topics[1].substring(26)}`.toLowerCase(),
                        };
                    }
                }
                else if (!TransferTX && log.topics.includes(ERC_TOKEN_APPROVAL_EVENT)) {
                    tokenTx = {
                        tokenType: log.topics[3] ? types_1.TransactionType.ERC_721 : types_1.TransactionType.ERC_20,
                        tokenFrom: `0x${log.topics[1].substring(26)}`.toLowerCase(),
                        tokenTo: `0x${log.topics[2].substring(26)}`.toLowerCase(),
                        tokenValue: log.topics[3] || log.data,
                        tokenEvent: 'Approval',
                    };
                }
                else if (log.topics.includes(UNISWAP_SYNC_EVENT)) {
                    if (index_1.config.verbose)
                        console.log('Uniswap Sync', log.data.length, log.data);
                    if (index_1.config.verbose)
                        if (log.data.length == 130) {
                            console.log(log.data.substring(0, 66));
                            console.log(web3_1.default.utils.fromWei(`${log.data.substring(0, 66)}`, 'ether'));
                            console.log(log.data.substring(66, 130));
                            console.log(web3_1.default.utils.fromWei(`${log.data.substring(66, 130)}`, 'ether'));
                        }
                        else
                            console.log('length is not equal to 130');
                }
                else if (log.topics.includes(UNISWAP_SWAP_EVENT)) {
                    if (index_1.config.verbose)
                        console.log('Uniswap Swap', log.data.length, log.data);
                    if (index_1.config.verbose)
                        if (log.data.length == 258) {
                            console.log(log.data.substring(0, 66));
                            console.log(web3_1.default.utils.fromWei(`${log.data.substring(0, 66)}`, 'ether'));
                            console.log(log.data.substring(66, 130));
                            console.log(web3_1.default.utils.fromWei(`${log.data.substring(66, 130)}`, 'ether'));
                            console.log(log.data.substring(130, 194));
                            console.log(web3_1.default.utils.fromWei(`${log.data.substring(130, 194)}`, 'ether'));
                            console.log(log.data.substring(194, 258));
                            console.log(web3_1.default.utils.fromWei(`${log.data.substring(192, 258)}`, 'ether'));
                        }
                        else
                            console.log('length is not equal to 258');
                }
                else if (log.topics.includes(UNISWAP_DEPOSIT_EVENT)) {
                    if (index_1.config.verbose)
                        console.log('Uniswap Deposit', log.data.length, log.data);
                    if (index_1.config.verbose)
                        console.log(web3_1.default.utils.fromWei(log.data, 'ether'));
                    if (tx.txTo !== log.address)
                        tokenTx = {
                            tokenType: types_1.TransactionType.EVM_Internal,
                            tokenFrom: `0x${log.topics[1].substring(26)}`.toLowerCase(),
                            tokenTo: log.address,
                            tokenValue: log.data,
                            tokenEvent: 'Internal Transfer',
                        };
                }
                else if (log.topics.includes(UNISWAP_WITHDRAWAL_EVENT)) {
                    if (index_1.config.verbose)
                        console.log('Uniswap Withdraw', log.data.length, log.data);
                    if (index_1.config.verbose)
                        console.log(web3_1.default.utils.fromWei(log.data, 'ether'));
                    if (tx.txTo !== log.address)
                        tokenTx = {
                            tokenType: types_1.TransactionType.EVM_Internal,
                            tokenFrom: log.address,
                            tokenTo: `0x${log.topics[1].substring(26)}`.toLowerCase(),
                            tokenValue: log.data,
                            tokenEvent: 'Internal Transfer',
                        };
                }
            }
            if (tokenTx) {
                txs.push({
                    ...tokenTx,
                    contractAddress: log.address,
                });
                if (tokenTx.tokenTo &&
                    tokenTx.tokenTo !== tx.txFrom &&
                    tokenTx.tokenTo !== tx.txTo &&
                    !accs.includes(tokenTx.tokenTo)) {
                    accs.push(tokenTx.tokenTo.toLowerCase());
                }
                // Skip saving ERC1155 tokens of an address for now
                if (tokenTx.tokenEvent !== 'Approval' &&
                    tokenTx.tokenEvent !== 'Approval For All' &&
                    (tokenTx.tokenType === types_1.TransactionType.EVM_Internal ||
                        tokenTx.tokenType === types_1.TransactionType.ERC_20 ||
                        tokenTx.tokenType === types_1.TransactionType.ERC_721)) {
                    const storageKey = tokenTx.tokenType === types_1.TransactionType.ERC_20
                        ? ERC_20_BALANCE_SLOT
                        : tokenTx.tokenType === types_1.TransactionType.ERC_721
                            ? ERC_721_BALANCE_SLOT
                            : ERC_1155_BALANCE_SLOT;
                    if (tokenTx.tokenFrom !== exports.ZERO_ETH_ADDRESS) {
                        let tokenValue = '0';
                        let calculatedKey = web3_1.default.utils.soliditySha3({ type: 'uint', value: tokenTx.tokenFrom }, { type: 'uint', value: storageKey });
                        let contractStorage = null;
                        if (Object.keys(storageKeyValueMap).length === 0 ||
                            !storageKeyValueMap[calculatedKey + log.address]) {
                            if (Object.keys(storageKeyValueMap).length === 0) {
                                const shardusAddress = log.address.slice(2).substring(0, 8) + (calculatedKey === null || calculatedKey === void 0 ? void 0 : calculatedKey.substring(8));
                                contractStorage = await (0, account_1.queryAccountByAccountId)(shardusAddress);
                                /* prettier-ignore */ if (index_1.config.verbose)
                                    console.log('contractStorage', contractStorage);
                            }
                            if (!contractStorage)
                                for (let i = 0; i < 20; i++) {
                                    calculatedKey = web3_1.default.utils.soliditySha3({ type: 'uint', value: tokenTx.tokenFrom }, { type: 'uint', value: '' + i });
                                    /* prettier-ignore */ if (index_1.config.verbose)
                                        console.log('calculatedKey', calculatedKey + log.address);
                                    if (Object.keys(storageKeyValueMap).length === 0) {
                                        const shardusAddress = log.address.slice(2).substring(0, 8) + (calculatedKey === null || calculatedKey === void 0 ? void 0 : calculatedKey.substring(8));
                                        contractStorage = await (0, account_1.queryAccountByAccountId)(shardusAddress);
                                        /* prettier-ignore */ if (index_1.config.verbose)
                                            console.log('contractStorage', contractStorage);
                                        break;
                                    }
                                    else if (storageKeyValueMap[calculatedKey + log.address])
                                        break;
                                }
                            /* prettier-ignore */ if (index_1.config.verbose)
                                console.log(tokenTx.tokenType, tokenTx.tokenFrom, calculatedKey + log.address);
                        }
                        if (storageKeyValueMap[calculatedKey + log.address] ||
                            (contractStorage && contractStorage.ethAddress === log.address)) {
                            /* prettier-ignore */ if (index_1.config.verbose)
                                console.log(storageKeyValueMap[calculatedKey + log.address]);
                            const value = contractStorage
                                ? Uint8Array.from(Object.values(contractStorage.account['value']))
                                : Uint8Array.from(Object.values(storageKeyValueMap[calculatedKey + log.address].value));
                            try {
                                const decode = rlp_1.RLP.decode(value);
                                // if (tokenTx.tokenType === TransactionType.ERC_20) {
                                //   tokenValue = Web3.utils.fromWei(decode, 'ether')
                                // } else if (tokenTx.tokenType === TransactionType.ERC_721) {
                                //   tokenValue = Web3.utils.hexToNumberString('0x' + decode)
                                // }
                                // /* prettier-ignore */ if (config.verbose) console.log('decode', decode)
                                // tokenValue = '0x' + decode // Seems we can use this as well; but it needs some adaptive changes when decoding in the frontend
                                tokenValue = (decode === null || decode === void 0 ? void 0 : decode.length) > 0 ? web3_1.default.utils.hexToNumberString((0, util_1.bytesToHex)(decode)) : '0';
                            }
                            catch (e) {
                                console.error('Error in decoding tokenValue from contract Storage', e);
                                tokenValue = '0';
                            }
                            /* prettier-ignore */ if (index_1.config.verbose)
                                console.log(calculatedKey, tokenValue);
                        }
                        tokens.push({
                            ethAddress: tokenTx.tokenFrom,
                            contractAddress: log.address,
                            tokenValue,
                            tokenType: tokenTx.tokenType,
                        });
                    }
                    if (tokenTx.tokenTo !== exports.ZERO_ETH_ADDRESS) {
                        let tokenValue = '0';
                        let calculatedKey = web3_1.default.utils.soliditySha3({ type: 'uint', value: tokenTx.tokenTo }, { type: 'uint', value: storageKey });
                        /* prettier-ignore */ if (index_1.config.verbose)
                            console.log(tokenTx.tokenType, tokenTx.tokenTo, calculatedKey + log.address);
                        let contractStorage = null;
                        if (Object.keys(storageKeyValueMap).length === 0 ||
                            !storageKeyValueMap[calculatedKey + log.address]) {
                            if (Object.keys(storageKeyValueMap).length === 0) {
                                const shardusAddress = log.address.slice(2).substring(0, 8) + (calculatedKey === null || calculatedKey === void 0 ? void 0 : calculatedKey.substring(8));
                                contractStorage = await (0, account_1.queryAccountByAccountId)(shardusAddress);
                                /* prettier-ignore */ if (index_1.config.verbose)
                                    console.log('contractStorage', contractStorage);
                            }
                            if (!contractStorage)
                                for (let i = 0; i < 20; i++) {
                                    calculatedKey = web3_1.default.utils.soliditySha3({ type: 'uint', value: tokenTx.tokenTo }, { type: 'uint', value: '' + i });
                                    /* prettier-ignore */ if (index_1.config.verbose)
                                        console.log('calculatedKey', calculatedKey + log.address);
                                    if (Object.keys(storageKeyValueMap).length === 0) {
                                        const shardusAddress = log.address.slice(2).substring(0, 8) + (calculatedKey === null || calculatedKey === void 0 ? void 0 : calculatedKey.substring(8));
                                        contractStorage = await (0, account_1.queryAccountByAccountId)(shardusAddress);
                                        /* prettier-ignore */ if (index_1.config.verbose)
                                            console.log('contractStorage', contractStorage);
                                        break;
                                    }
                                    else if (storageKeyValueMap[calculatedKey + log.address] &&
                                        storageKeyValueMap[calculatedKey + log.address].ethAddress === log.address)
                                        break;
                                }
                            /* prettier-ignore */ if (index_1.config.verbose)
                                console.log(tokenTx.tokenType, tokenTx.tokenTo, calculatedKey + log.address);
                        }
                        if (storageKeyValueMap[calculatedKey + log.address] ||
                            (contractStorage && contractStorage.ethAddress === log.address)) {
                            /* prettier-ignore */ if (index_1.config.verbose)
                                console.log(storageKeyValueMap[calculatedKey + log.address].value);
                            const value = contractStorage
                                ? Uint8Array.from(Object.values(contractStorage.account['value']))
                                : Uint8Array.from(Object.values(storageKeyValueMap[calculatedKey + log.address].value));
                            try {
                                const decode = rlp_1.RLP.decode(value);
                                // if (tokenTx.tokenType === TransactionType.ERC_20) {
                                //   tokenValue = Web3.utils.fromWei(decode, 'ether')
                                // } else if (tokenTx.tokenType === TransactionType.ERC_721) {
                                //   tokenValue = Web3.utils.hexToNumberString('0x' + decode)
                                // }
                                // /* prettier-ignore */ if (config.verbose) console.log('decode', decode)
                                // tokenValue = '0x' + decode // Seems we can use this as well; but it needs some adaptive changes when decoding in the frontend
                                tokenValue = (decode === null || decode === void 0 ? void 0 : decode.length) > 0 ? web3_1.default.utils.hexToNumberString((0, util_1.bytesToHex)(decode)) : '0';
                            }
                            catch (e) {
                                console.error('Error in decoding tokenValue from contract Storage', e);
                                tokenValue = '0';
                            }
                            /* prettier-ignore */ if (index_1.config.verbose)
                                console.log(calculatedKey, tokenValue);
                        }
                        if (tokenValue !== '0') {
                            tokens.push({
                                ethAddress: tokenTx.tokenTo,
                                contractAddress: log.address,
                                tokenValue,
                                tokenType: tokenTx.tokenType,
                            });
                        }
                    }
                }
            }
        }
    }
    if (!index_1.config.processData.decodeTokenTransfer)
        return { txs, accs, tokens };
    const data = 'readableReceipt' in tx.wrappedEVMAccount ? (_e = tx.wrappedEVMAccount.readableReceipt) === null || _e === void 0 ? void 0 : _e.data : '';
    const methodCode = data && data.length > 10 ? data.substring(0, 10) : null;
    // eslint-disable-next-line security/detect-object-injection
    if (methodCode && ERC_TOKEN_METHOD_DIC[methodCode] === 'Disperse ETH') {
        try {
            const web3 = new web3_1.default();
            const result = web3.eth.abi.decodeParameters(['address[]', 'uint256[]'], 'readableReceipt' in tx.wrappedEVMAccount
                ? ((_f = tx.wrappedEVMAccount.readableReceipt) === null || _f === void 0 ? void 0 : _f.data.slice(10)) || ''
                : '');
            if ((result === null || result === void 0 ? void 0 : result['0']) &&
                result['1'] &&
                result['0'].length === result['1'].length) {
                for (let i = 0; i < result['0'].length; i++) {
                    const tokenTx = {
                        tokenType: types_1.TransactionType.EVM_Internal,
                        tokenFrom: tx.txTo,
                        /* eslint-disable security/detect-object-injection */
                        tokenTo: result['0'][i].toLowerCase(),
                        tokenValue: result['1'][i],
                        /* eslint-enable security/detect-object-injection */
                        tokenEvent: 'Internal Transfer',
                    };
                    txs.push({
                        ...tokenTx,
                        contractAddress: tx.txTo,
                    });
                    accs.push(tokenTx.tokenTo.toLowerCase());
                }
            }
        }
        catch (e) {
            console.log('Error in decoding disperseETH', e);
        }
    }
    const decodeTxResult = { txs, accs, tokens };
    return decodeTxResult;
};
exports.decodeTx = decodeTx;
const getContractInfo = async (contractAddress) => {
    let contractType = types_1.ContractType.GENERIC;
    const contractInfo = {};
    let foundCorrectContract = false;
    try {
        const web3 = (await (0, transaction_1.getWeb3)());
        const Token = new web3.eth.Contract(ERC20_json_1.default.abi, contractAddress);
        contractInfo.name = await Token.methods.name().call();
        if (index_1.config.verbose)
            console.log('Token Name', contractInfo.name);
        contractInfo.symbol = await Token.methods.symbol().call();
        contractInfo.totalSupply = await Token.methods.totalSupply().call();
        contractInfo.decimals = await Token.methods.decimals().call();
        foundCorrectContract = true;
        contractType = types_1.ContractType.ERC_20;
        // await sleep(200); // Awaiting a bit to refresh the service points of the validator
    }
    catch (e) {
        // console.log(e);
        console.log('Non ERC 20 Contract', contractAddress); // It could be not ERC 20 Contract
        // await sleep(100); // Awaiting a bit to refresh the service points of the validator
    }
    if (!foundCorrectContract) {
        try {
            const web3 = (await (0, transaction_1.getWeb3)());
            const Token = new web3.eth.Contract(ERC721_json_1.default.abi, contractAddress);
            const result = await Token.methods.supportsInterface(ERC_721_INTERFACE).call();
            if (result) {
                if (!contractInfo.name)
                    contractInfo.name = await Token.methods.name().call();
                if (index_1.config.verbose)
                    console.log('ERC 721 Name', contractInfo.name);
                if (!contractInfo.symbol)
                    contractInfo.symbol = await Token.methods.symbol().call();
                foundCorrectContract = true;
                contractType = types_1.ContractType.ERC_721;
            }
        }
        catch (e) {
            console.log('Non ERC 721 Contract', contractAddress); // It could be not ERC 20 Contract
        }
    }
    if (!foundCorrectContract) {
        try {
            const web3 = (await (0, transaction_1.getWeb3)());
            const Token = new web3.eth.Contract(ERC1155_json_1.default.abi, contractAddress);
            const result = await Token.methods.supportsInterface(ERC_1155_INTERFACE).call();
            if (result) {
                foundCorrectContract = true;
                contractType = types_1.ContractType.ERC_1155;
            }
        }
        catch (e) {
            console.log('Non ERC 1155 Contract', contractAddress); // It could be not ERC 20 Contract
        }
    }
    return { contractInfo, contractType };
};
exports.getContractInfo = getContractInfo;
//# sourceMappingURL=TxDecoder.js.map