"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateData = void 0;
const crypto = __importStar(require("@shardeum-foundation/lib-crypto-utils"));
const utils = __importStar(require("../utils"));
const config_1 = require("../config");
const cycle_1 = require("../storage/cycle");
const receipt_1 = require("../storage/receipt");
const originalTxData_1 = require("../storage/originalTxData");
const DataLogWriter_1 = require("./DataLogWriter");
const block_1 = require("../storage/block");
const lib_types_1 = require("@shardeum-foundation/lib-types");
async function validateData(data) {
    let err = utils.validateTypes(data, {
        sign: 'o',
        receipt: 'o?',
        cycle: 'o?',
        originalTx: 'o?',
    });
    if (err) {
        console.error('Data received from distributor failed validation', err);
        return false;
    }
    err = utils.validateTypes(data.sign, { owner: 's', sig: 's' });
    if (err) {
        return false;
    }
    if (data.sign.owner !== config_1.config.distributorInfo.publicKey) {
        console.error('Data received from distributor has invalid key');
        return false;
    }
    if (!crypto.verifyObj(data)) {
        console.error('Data received from distributor has invalid signature');
        return false;
    }
    if (!data.receipt && !data.cycle && !data.originalTx) {
        console.error('Data received from distributor is invalid', data);
        return false;
    }
    if (data.receipt) {
        DataLogWriter_1.ReceiptLogWriter.writeToLog(`${lib_types_1.Utils.safeStringify(data.receipt)}\n`);
        await (0, receipt_1.processReceiptData)([data.receipt]);
        return true;
    }
    if (data.cycle) {
        DataLogWriter_1.CycleLogWriter.writeToLog(`${lib_types_1.Utils.safeStringify(data.cycle)}\n`);
        await (0, cycle_1.insertOrUpdateCycle)(data.cycle);
        // optimistically upsert blocks for next cycle if it is wrong, it will be corrected in next cycle
        await (0, block_1.upsertBlocksForCycleCore)(data.cycle.counter + 1, data.cycle.cycleRecord.start + config_1.config.blockIndexing.cycleDurationInSeconds);
        return true;
    }
    if (data.originalTx) {
        DataLogWriter_1.OriginalTxDataLogWriter.writeToLog(`${lib_types_1.Utils.safeStringify(data.originalTx)}\n`);
        await (0, originalTxData_1.processOriginalTxData)([data.originalTx]);
        return true;
    }
    return false;
}
exports.validateData = validateData;
//# sourceMappingURL=validateData.js.map