"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startServer = void 0;
const dotenv = __importStar(require("dotenv"));
dotenv.config();
const ws_1 = __importDefault(require("ws"));
const lib_types_1 = require("@shardeum-foundation/lib-types");
const Storage = __importStar(require("./storage"));
const Crypto = __importStar(require("./utils/crypto"));
const cycle = __importStar(require("./storage/cycle"));
const receipt = __importStar(require("./storage/receipt"));
const originalTxData = __importStar(require("./storage/originalTxData"));
const DataSync_1 = require("./class/DataSync");
const validateData_1 = require("./class/validateData");
const types_1 = require("./types");
const DataLogWriter_1 = require("./class/DataLogWriter");
const CollectorSocketconnection_1 = require("./log_subscription/CollectorSocketconnection");
// config variables
const config_1 = require("./config");
const utils_1 = require("./utils");
const rmq_cycles_1 = __importDefault(require("./collectors/rmq_cycles"));
const rmq_original_txs_1 = __importDefault(require("./collectors/rmq_original_txs"));
const rmq_receipts_1 = __importDefault(require("./collectors/rmq_receipts"));
if (process.env.PORT) {
    config_1.config.port.collector = process.env.PORT;
}
const DistributorFirehoseEvent = 'FIREHOSE';
let ws;
let reconnecting = false;
let connected = false;
const env = process.env;
const args = process.argv;
const path = require("path");
const fs = require("fs");
if (config_1.config.env == config_1.envEnum.DEV) {
    //default debug mode keys
    //  pragma: allowlist nextline secret
    config_1.config.USAGE_ENDPOINTS_KEY = 'ceba96f6eafd2ea59e68a0b0d754a939';
    config_1.config.collectorInfo.secretKey =
        //  pragma: allowlist nextline secret
        '7d8819b6fac8ba2fbac7363aaeb5c517e52e615f95e1a161d635521d5e4969739426b64e675cad739d69526bf7e27f3f304a8a03dca508a9180f01e9269ce447';
    config_1.config.collectorInfo.publicKey =
        // pragma: allowlist nextline secret
        '9426b64e675cad739d69526bf7e27f3f304a8a03dca508a9180f01e9269ce447';
    config_1.config.distributorInfo.publicKey =
        // pragma: allowlist nextline secret
        '758b1c119412298802cd28dbfa394cdfeecc4074492d60844cc192d632d84de3';
}
else {
    // Pull in secrets
    const secretsPath = path.join(__dirname, '../.secrets');
    const secrets = {};
    if (fs.existsSync(secretsPath)) {
        const lines = fs.readFileSync(secretsPath, 'utf-8').split('\n').filter(Boolean);
        lines.forEach((line) => {
            const [key, value] = line.split('=');
            secrets[key.trim()] = value.trim();
        });
    }
    if (secrets['USAGE_ENDPOINTS_KEY']) {
        config_1.config.USAGE_ENDPOINTS_KEY = secrets['USAGE_ENDPOINTS_KEY'];
    }
    if (secrets['COLLECTOR_SECRET_KEY']) {
        config_1.config.collectorInfo.secretKey = secrets['COLLECTOR_SECRET_KEY'];
    }
    if (secrets['COLLECTOR_PUBLIC_KEY']) {
        config_1.config.collectorInfo.publicKey = secrets['COLLECTOR_PUBLIC_KEY'];
    }
    if (secrets['DISTRIBUTOR_PUBLIC_KEY']) {
        config_1.config.distributorInfo.publicKey = secrets['DISTRIBUTOR_PUBLIC_KEY'];
    }
}
const startServer = async () => {
    console.log(`Collector Mode: ${config_1.config.collectorMode}`);
    (0, config_1.overrideDefaultConfig)(env, args);
    // Set crypto hash keys from config
    Crypto.setCryptoHashKey(config_1.config.hashKey);
    await Storage.initializeDB();
    Storage.addExitListeners(ws);
    // Check if there is any existing data in the db
    let lastStoredReceiptCount = await receipt.queryReceiptCount();
    let lastStoredOriginalTxDataCount = await originalTxData.queryOriginalTxDataCount();
    let lastStoredCycleCount = await cycle.queryCycleCount();
    let lastStoredCycle = (await cycle.queryLatestCycleRecords(1))[0];
    if (lastStoredCycleCount > 0 && lastStoredCycle.counter !== lastStoredCycleCount - 1) {
        console.log('lastStoredCycleCount', lastStoredCycleCount, 'lastStoredCycleCounter', lastStoredCycle.counter);
        // Check if the last stored cycle counter is correct
        throw Error('The last stored cycle counter does not match with the last stored cycle count! Patch the missing cycle data and start the server again!');
    }
    let totalReceiptsToSync = 0;
    let totalCyclesToSync = 0;
    let totalOriginalTxsToSync = 0;
    let lastStoredReceiptCycle = 0;
    let lastStoredOriginalTxDataCycle = 0;
    let response = await (0, DataSync_1.queryFromDistributor)(DataSync_1.DataType.TOTALDATA, {});
    if (response.data &&
        response.data.totalReceipts >= 0 &&
        response.data.totalCycles >= 0 &&
        response.data.totalOriginalTxs >= 0) {
        totalReceiptsToSync = response.data.totalReceipts;
        totalCyclesToSync = response.data.totalCycles;
        totalOriginalTxsToSync = response.data.totalOriginalTxs;
        console.log('totalReceiptsToSync', totalReceiptsToSync, 'totalCyclesToSync', totalCyclesToSync, 'totalOriginalTxsToSync', totalOriginalTxsToSync);
    }
    console.log('lastStoredReceiptCount', lastStoredReceiptCount, 'lastStoredCycleCount', lastStoredCycleCount, 'lastStoredOriginalTxDataCount', lastStoredOriginalTxDataCount);
    // Make sure the data that saved are authentic by comparing receipts count of last 10 cycles for receipts data, originalTxs count of last 10 cycles for originalTxData data and 10 last cycles for cycles data
    if (lastStoredReceiptCount > 0) {
        const lastStoredReceiptInfo = await receipt.queryLatestReceipts(1);
        if (lastStoredReceiptInfo && lastStoredReceiptInfo.length > 0)
            lastStoredReceiptCycle = lastStoredReceiptInfo[0].cycle;
        const receiptResult = await (0, DataSync_1.compareWithOldReceiptsData)(lastStoredReceiptCycle);
        if (!receiptResult.success) {
            throw Error('The last saved receipts of last 10 cycles data do not match with the distributor data! Clear the DB and start the server again!');
        }
        lastStoredReceiptCycle = receiptResult.matchedCycle;
    }
    if (lastStoredOriginalTxDataCount > 0) {
        const lastStoredOriginalTxDataInfo = await originalTxData.queryOriginalTxsData(1);
        if (lastStoredOriginalTxDataInfo && lastStoredOriginalTxDataInfo.length > 0)
            lastStoredOriginalTxDataCycle = lastStoredOriginalTxDataInfo[0].cycle;
        const originalTxResult = await (0, DataSync_1.compareWithOldOriginalTxsData)(lastStoredOriginalTxDataCycle);
        if (!originalTxResult.success) {
            throw Error('The last saved originalTxsData of last 10 cycles data do not match with the distributor data! Clear the DB and start the server again!');
        }
        lastStoredOriginalTxDataCycle = originalTxResult.matchedCycle;
    }
    if (totalCyclesToSync > lastStoredCycleCount && lastStoredCycleCount > 10) {
        const cycleResult = await (0, DataSync_1.compareWithOldCyclesData)(lastStoredCycleCount);
        if (!cycleResult.success) {
            throw Error('The last saved 10 cycles data does not match with the distributor data! Clear the DB and start the server again!');
        }
        lastStoredCycleCount = cycleResult.cycle;
    }
    if (lastStoredReceiptCount > 0 || lastStoredOriginalTxDataCount > 0) {
        if (lastStoredReceiptCount > totalReceiptsToSync) {
            throw Error('The existing db has more receipts data than the network data! Clear the DB and start the server again!');
        }
        if (lastStoredOriginalTxDataCount > totalOriginalTxsToSync) {
            throw Error('The existing db has more originalTxsData data than the network data! Clear the DB and start the server again!');
        }
    }
    if (config_1.config.dataLogWrite)
        await (0, DataLogWriter_1.initDataLogWriter)();
    if (config_1.config.enableCollectorSocketServer)
        (0, CollectorSocketconnection_1.setupCollectorSocketServer)();
    addSigListeners();
    if (config_1.config.collectorMode === config_1.collectorMode.MQ) {
        startRMQEventsConsumers();
    }
    else {
        const CONNECT_TO_DISTRIBUTOR_MAX_RETRY = 10;
        let retry = 0;
        // Connect to the distributor
        while (!connected) {
            connectToDistributor();
            retry++;
            await (0, utils_1.sleep)(2000);
            if (!connected && retry > CONNECT_TO_DISTRIBUTOR_MAX_RETRY) {
                throw Error('Cannot connect to the distributor!');
            }
        }
    }
    // If there is already some data in the db, we can assume that the genesis accounts data has been synced already
    if (lastStoredCycleCount === 0)
        await (0, DataSync_1.downloadAndSyncGenesisAccounts)(); // To sync accounts data that are from genesis accounts/accounts data that the network start with
    // Refresh the total data to sync after collector connected to distributor
    response = await (0, DataSync_1.queryFromDistributor)(DataSync_1.DataType.TOTALDATA, {});
    if (response.data &&
        response.data.totalReceipts >= 0 &&
        response.data.totalCycles >= 0 &&
        response.data.totalOriginalTxs >= 0) {
        totalReceiptsToSync = response.data.totalReceipts;
        totalCyclesToSync = response.data.totalCycles;
        totalOriginalTxsToSync = response.data.totalOriginalTxs;
        console.log('totalReceiptsToSync', totalReceiptsToSync, 'totalCyclesToSync', totalCyclesToSync, 'totalOriginalTxsToSync', totalOriginalTxsToSync);
    }
    if (totalReceiptsToSync > lastStoredReceiptCount)
        (0, DataSync_1.toggleNeedSyncing)();
    if (!DataSync_1.needSyncing && totalOriginalTxsToSync > lastStoredOriginalTxDataCount)
        (0, DataSync_1.toggleNeedSyncing)();
    if (!DataSync_1.needSyncing && totalCyclesToSync > lastStoredCycleCount)
        (0, DataSync_1.toggleNeedSyncing)();
    if (!DataSync_1.needSyncing)
        return;
    console.log(lastStoredReceiptCount, totalReceiptsToSync, lastStoredCycleCount, totalCyclesToSync, lastStoredOriginalTxDataCount, totalOriginalTxsToSync);
    // Sync receipts and originalTxsData data first if there is old data
    if (lastStoredReceiptCycle > 0 &&
        totalCyclesToSync > lastStoredReceiptCycle &&
        totalReceiptsToSync > lastStoredReceiptCount) {
        await (0, DataSync_1.downloadReceiptsBetweenCycles)(lastStoredReceiptCycle, totalCyclesToSync);
        lastStoredReceiptCount = await receipt.queryReceiptCount();
    }
    if (lastStoredOriginalTxDataCycle > 0 &&
        totalCyclesToSync > lastStoredOriginalTxDataCycle &&
        totalOriginalTxsToSync > lastStoredOriginalTxDataCount) {
        await (0, DataSync_1.downloadOriginalTxsDataBetweenCycles)(lastStoredOriginalTxDataCycle, totalCyclesToSync);
        lastStoredOriginalTxDataCount = await originalTxData.queryOriginalTxDataCount();
    }
    await (0, DataSync_1.downloadTxsDataAndCycles)(totalReceiptsToSync, lastStoredReceiptCount, totalOriginalTxsToSync, lastStoredOriginalTxDataCount, totalCyclesToSync, lastStoredCycleCount);
    (0, DataSync_1.toggleNeedSyncing)();
};
exports.startServer = startServer;
const attemptReconnection = () => {
    console.log(`Re-connecting Distributor in ${config_1.config.RECONNECT_INTERVAL_MS / 1000}s...`);
    reconnecting = true;
    setTimeout(connectToDistributor, config_1.config.RECONNECT_INTERVAL_MS);
};
const connectToDistributor = () => {
    const collectorInfo = {
        subscriptionType: DistributorFirehoseEvent,
        timestamp: Date.now(),
    };
    const queryString = encodeURIComponent(lib_types_1.Utils.safeStringify(Crypto.sign({ collectorInfo, sender: config_1.config.collectorInfo.publicKey })));
    const URL = `${config_1.DISTRIBUTOR_URL}?data=${queryString}`;
    ws = new ws_1.default(URL);
    ws.onopen = () => {
        console.log(`✅ Socket connected to the Distributor @ ${config_1.config.distributorInfo.ip}:${config_1.config.distributorInfo.port}}`);
        connected = true;
        reconnecting = false;
    };
    // Listening to messages from the server (child process)
    ws.on('message', (data) => {
        try {
            (0, validateData_1.validateData)(lib_types_1.Utils.safeJsonParse(data));
        }
        catch (e) {
            console.log('Error in processing received data!', e);
        }
    });
    ws.onerror = (error) => {
        console.error('Distributor WebSocket error:', error.message);
        reconnecting = false;
    };
    // Listening to Socket termination event from the Distributor
    ws.onclose = (closeEvent) => {
        console.log('❌ Connection with Server Terminated!.');
        switch (closeEvent.code) {
            case types_1.DistributorSocketCloseCodes.DUPLICATE_CONNECTION_CODE:
                console.log('❌ Socket Connection w/ same client credentials attempted. Dropping existing connection.');
                break;
            case types_1.DistributorSocketCloseCodes.SUBSCRIBER_EXPIRATION_CODE:
                console.log('❌ Subscription Validity Expired. Connection Terminated.');
                break;
            default:
                console.log(`❌ Socket Connection w/ Distributor Terminated with code: ${closeEvent.code}`);
                reconnecting = false;
                break;
        }
        if (!reconnecting)
            attemptReconnection();
    };
};
// start queue consumers for cycles, transactions and receipts events
const startRMQEventsConsumers = () => {
    const rmqCyclesConsumer = new rmq_cycles_1.default();
    const rmqTransactionsConsumer = new rmq_original_txs_1.default();
    const rmqReceiptsConsumer = new rmq_receipts_1.default();
    rmqCyclesConsumer.start();
    rmqTransactionsConsumer.start();
    rmqReceiptsConsumer.start();
    // add signal listeners
    process.on('SIGTERM', async () => {
        console.log(`Initiated RabbitMQ connections cleanup`);
        await rmqCyclesConsumer.cleanUp();
        await rmqTransactionsConsumer.cleanUp();
        await rmqReceiptsConsumer.cleanUp();
        console.log(`Completed RabbitMQ connections cleanup`);
    });
    process.on('SIGINT', async () => {
        console.log(`Initiated RabbitMQ connections cleanup`);
        await rmqCyclesConsumer.cleanUp();
        await rmqTransactionsConsumer.cleanUp();
        await rmqReceiptsConsumer.cleanUp();
        console.log(`Completed RabbitMQ connections cleanup`);
    });
};
const addSigListeners = () => {
    process.on('SIGUSR1', async () => {
        console.log('DETECTED SIGUSR1 SIGNAL');
        // Reload the config.json
        (0, config_1.overrideDefaultConfig)(env, args);
        console.log('Config reloaded', config_1.config);
    });
    console.log('Registerd signal listeners.');
};
(0, exports.startServer)();
process.on('uncaughtException', (error) => {
    console.error('Uncaught Exception in Collector: ', error);
});
//# sourceMappingURL=collector.js.map