"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const consumer_1 = __importDefault(require("../messaging/rabbitmq/consumer"));
const validateData_1 = require("../class/validateData");
const lib_types_1 = require("@shardeum-foundation/lib-types");
class RMQCyclesConsumer {
    constructor() {
        const queueName = process.env.RMQ_CYCLES_QUEUE_NAME;
        if (queueName === null || queueName === undefined || queueName.trim() === '') {
            throw new Error('[RMQCyclesConsumer]: please provide queue name for consumer');
        }
        this.consumer = new consumer_1.default('Cycles', queueName, 20, this.processMessage.bind(this));
    }
    async start() {
        await this.consumer.consume();
    }
    async cleanUp() {
        await this.consumer.cleanUp();
    }
    async processMessage(msgStr) {
        try {
            const success = await (0, validateData_1.validateData)(lib_types_1.Utils.safeJsonParse(msgStr));
            console.log(`cycles#processMessage: ${success}`);
            return success;
        }
        catch (e) {
            console.error(`[RMQCyclesConsumer]: Error while processing message: ${e}`);
            return false;
        }
    }
}
exports.default = RMQCyclesConsumer;
//# sourceMappingURL=rmq_cycles.js.map