"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DISTRIBUTOR_URL = exports.config = exports.overrideDefaultConfig = exports.collectorMode = exports.envEnum = void 0;
/* eslint-disable security/detect-object-injection */
const fs_1 = require("fs");
const deepmerge_1 = __importDefault(require("deepmerge"));
const minimist_1 = __importDefault(require("minimist"));
const path_1 = require("path");
exports.envEnum = {
    DEV: 'development',
    PROD: 'production',
};
var collectorMode;
(function (collectorMode) {
    collectorMode["WS"] = "WS";
    collectorMode["MQ"] = "MQ";
})(collectorMode = exports.collectorMode || (exports.collectorMode = {}));
let config = {
    env: process.env.SHARDEUM_COLLECTOR_MODE || exports.envEnum.PROD,
    host: process.env.HOST || '127.0.0.1',
    dbPath: process.env.COLLECTOR_DB_PATH || "db.sqlite3",
    dataLogWrite: false,
    dataLogWriter: {
        dirName: 'data-logs',
        maxLogFiles: 10,
        maxReceiptEntries: 1000,
        maxCycleEntries: 1000,
        maxOriginalTxEntries: 1000, // This value should be equivalent to the max TPS experiened by the network.
    },
    collectorInfo: {
        publicKey: '',
        secretKey: '',
    },
    hashKey: '69fa4195670576c0160d660c3be36556ff8d504725be8a59b5a96509e0c994bc',
    enableCollectorSocketServer: Boolean(process.env.ENABLE_COLLECTOR_SOCKET_SERVER) || false,
    port: {
        collector: process.env.COLLECTOR_PORT || '4444',
        server: process.env.PORT || '6101',
        log_server: process.env.LOG_SERVER_PORT || '4446',
    },
    distributorInfo: {
        ip: process.env.DISTRIBUTOR_IP || '127.0.0.1',
        port: process.env.DISTRIBUTOR_PORT || '6100',
        publicKey: '',
    },
    rpcUrl: 'http://127.0.0.1:8080',
    apiUrl: '',
    verbose: false,
    fastifyDebugLog: false,
    rateLimit: 100,
    patchData: false,
    USAGE_ENDPOINTS_KEY: '',
    RECONNECT_INTERVAL_MS: 10000,
    processData: {
        indexReceipt: true,
        indexOriginalTxData: false,
        decodeContractInfo: false,
        decodeTokenTransfer: false,
    },
    enableTxHashCache: false,
    findTxHashInOriginalTx: false,
    enableShardeumIndexer: true,
    shardeumIndexerSqlitePath: process.env.SERVICE_VALIDATOR_DB_PATH || "db.sqlite3",
    blockIndexing: {
        enabled: true,
        blockProductionRate: 6,
        initBlockNumber: 0,
        cycleDurationInSeconds: 60,
        latestBehindBySeconds: 10,
    },
    blockCache: {
        enabled: false,
        cacheUpdateIntervalInMillis: 5000,
    },
    saveAccountHistoryState: true,
    collectorMode: process.env.COLLECTOR_MODE || collectorMode.WS.toString(),
    storeReceiptBeforeStates: true,
};
exports.config = config;
let DISTRIBUTOR_URL = `http://${config.distributorInfo.ip}:${config.distributorInfo.port}`;
exports.DISTRIBUTOR_URL = DISTRIBUTOR_URL;
// Override default config params from config file, env vars, and cli args
function overrideDefaultConfig(env, args) {
    const file = (0, path_1.join)(process.cwd(), 'config.json');
    // Override config from config file
    try {
        // eslint-disable-next-line security/detect-non-literal-fs-filename
        const fileConfig = JSON.parse((0, fs_1.readFileSync)(file, { encoding: 'utf8' }));
        const overwriteMerge = (target, source) => source;
        exports.config = config = (0, deepmerge_1.default)(config, fileConfig, { arrayMerge: overwriteMerge });
    }
    catch (err) {
        if (err && err.code !== 'ENOENT') {
            console.warn('Failed to parse config file:', err);
        }
    }
    // Override config from env vars
    for (const param in config) {
        if (env[param]) {
            switch (typeof config[param]) {
                case 'number': {
                    config[param] = Number(env[param]);
                    break;
                }
                case 'string': {
                    config[param] = String(env[param]);
                    break;
                }
                case 'object': {
                    try {
                        const parameterStr = env[param];
                        if (parameterStr) {
                            const parameterObj = JSON.parse(parameterStr);
                            config[param] = parameterObj;
                        }
                    }
                    catch (e) {
                        console.error(e);
                        console.error('Unable to JSON parse', env[param]);
                    }
                    break;
                }
                case 'boolean': {
                    config[param] = String(env[param]).toLowerCase() === 'true';
                    break;
                }
            }
        }
    }
    // Override config from cli args
    const parsedArgs = (0, minimist_1.default)(args.slice(2));
    for (const param of Object.keys(config)) {
        if (parsedArgs[param]) {
            switch (typeof config[param]) {
                case 'number': {
                    config[param] = Number(parsedArgs[param]);
                    break;
                }
                case 'string': {
                    config[param] = String(parsedArgs[param]);
                    break;
                }
                case 'boolean': {
                    if (typeof parsedArgs[param] === 'boolean') {
                        config[param] = parsedArgs[param];
                    }
                    else {
                        config[param] = String(parsedArgs[param]).toLowerCase() === 'true';
                    }
                    break;
                }
            }
        }
    }
    exports.DISTRIBUTOR_URL = DISTRIBUTOR_URL = `http://${config.distributorInfo.ip}:${config.distributorInfo.port}`;
}
exports.overrideDefaultConfig = overrideDefaultConfig;
//# sourceMappingURL=index.js.map