"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const rate_limit_1 = __importDefault(require("@fastify/rate-limit"));
const websocket_1 = __importDefault(require("@fastify/websocket"));
const crypto = __importStar(require("crypto"));
const fastify_1 = __importDefault(require("fastify"));
const config_1 = require("./config");
const CollectorListener_1 = require("./log_subscription/CollectorListener");
const Handler_1 = require("./log_subscription/Handler");
const SocketManager_1 = require("./log_subscription/SocketManager");
const Storage = __importStar(require("./storage"));
const lib_types_1 = require("@shardeum-foundation/lib-types");
const start = async () => {
    // Init dependencies
    await Storage.initializeDB();
    Storage.addExitListeners();
    await (0, CollectorListener_1.setupCollectorListener)();
    // Init server
    const server = (0, fastify_1.default)({
        logger: false,
    });
    // Register plugins and middleware
    await server.register(websocket_1.default, {
        errorHandler: (error, connection, request, reply) => {
            server.log.error(`Error processing websocket request ${request.id}. Error ${error}`);
            reply.send({ error: error.message });
            connection.destroy(error);
        },
    });
    await server.register(rate_limit_1.default, {
        max: config_1.config.rateLimit,
        timeWindow: '1 minute',
        allowList: ['127.0.0.1', 'localhost'],
    });
    server.setErrorHandler((error, request, reply) => {
        server.log.error(`Error processing request ${request.id}. Error ${error}`);
        reply.send({ error: error.message });
    });
    // Register handler
    server.get('/evm_log_subscription', { websocket: true }, evmLogSubscriptionController);
    // Start server
    server.listen({
        port: Number(config_1.config.port.log_server),
        host: '0.0.0.0',
    }, async (err) => {
        if (err) {
            server.log.error(`Error starting Log server on port ${config_1.config.port.log_server}. Error ${err}`);
            throw err;
        }
        console.log('Log Server is listening on port:', config_1.config.port.log_server);
    });
};
const evmLogSubscriptionController = (connection) => {
    let socketId = crypto.randomBytes(32).toString('hex');
    socketId = crypto.createHash('sha256').update(socketId).digest().toString('hex');
    connection.socket.on('message', (message) => {
        try {
            const payload = lib_types_1.Utils.safeJsonParse(message.toString());
            Handler_1.evmLogSubscriptionHandler.onMessage(connection, payload, socketId);
            return;
        }
        catch (e) {
            connection.socket.send(lib_types_1.Utils.safeStringify({ error: e.message }));
            return;
        }
    });
    connection.socket.on('close', () => {
        try {
            (0, SocketManager_1.removeLogSubscriptionBySocketId)(socketId);
        }
        catch (e) {
            console.error(e);
        }
    });
};
start();
//# sourceMappingURL=log_server.js.map