"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupCollectorListener = void 0;
const socketClient = __importStar(require("socket.io-client"));
const config_1 = require("../config");
const CollectorDataParser_1 = require("./CollectorDataParser");
const SocketManager_1 = require("./SocketManager");
const CollectorSocketconnection_1 = require("./CollectorSocketconnection");
const setupCollectorListener = async () => {
    const socket = socketClient.connect(`http://${config_1.config.host}:${config_1.config.port.collector}`, {
        reconnection: true,
        reconnectionAttempts: 10,
    });
    // Register default socket event handlers
    socket.on('connect', () => console.log('Connected to collector'));
    socket.on('disconnect', () => console.log('Disconnected from collector'));
    socket.on('error', (err) => console.log(`Error from collector: ${err}`));
    // Register custom socket event handlers
    socket.on(CollectorSocketconnection_1.CycleDataWsEvent, cycleDataHandler);
    socket.on(CollectorSocketconnection_1.ReceiptDataWsEvent, receiptDataHandler);
};
exports.setupCollectorListener = setupCollectorListener;
const cycleDataHandler = async (data) => {
    /*prettier-ignore*/ console.log('Received archived cycle data, valid? ', data && data[0] && data[0].cycleRecord && data[0].cycleRecord.counter);
    console.log(`Cycle data: ${JSON.stringify(data, null, 2)}`);
};
const receiptDataHandler = async (data) => {
    /*prettier-ignore*/ console.log('Received receipt data from archiver.');
    const logs = (0, CollectorDataParser_1.extractLogsFromReceipts)(data);
    console.log(`Number of logs found in receipts: ${logs.length}`);
    if (logs.length == 0) {
        return;
    }
    const indexedLogs = new CollectorDataParser_1.IndexedLogs(logs);
    for (const [subscriptionId, subscription] of SocketManager_1.logSubscriptionMap.entries()) {
        const filteredLogs = indexedLogs.filter(subscription.filterOptions);
        console.log(`Number of logs found for subscription ${subscriptionId}: ${filteredLogs.length}`);
        if (filteredLogs.length > 0) {
            const socket = (0, SocketManager_1.getLogSocketClient)(subscription.socketId);
            if (socket) {
                console.log(`Sending ${filteredLogs.length} logs to socket ${subscription.socketId}`);
                await socket.socket.send(JSON.stringify({ method: 'log_found', subscription_id: subscriptionId, logs: filteredLogs }));
            }
        }
    }
};
//# sourceMappingURL=CollectorListener.js.map