"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.forwardReceiptData = exports.forwardCycleData = exports.setupCollectorSocketServer = exports.ReceiptDataWsEvent = exports.CycleDataWsEvent = void 0;
const socket_io_1 = require("socket.io");
const config_1 = require("../config");
const lib_types_1 = require("@shardeum-foundation/lib-types");
// constants
const ConnectionEvent = 'connection';
const DisconnectionEvent = 'disconnect';
const ErrorEvent = 'error';
exports.CycleDataWsEvent = '/data/cycle';
exports.ReceiptDataWsEvent = '/data/receipt';
const registeredLogServers = new Map();
const setupCollectorSocketServer = () => {
    const socketServer = new socket_io_1.Server();
    socketServer.on(ConnectionEvent, (socket) => {
        console.log(`New LogServer registered ${socket.id}`);
        registeredLogServers.set(socket.id, socket);
        socket.on(DisconnectionEvent, () => {
            console.log(`LogServer ${socket.id} disconnected`);
            registeredLogServers.delete(socket.id);
        });
        socket.on(ErrorEvent, (err) => {
            console.log(`LogServer ${socket.id} error: ${err}. Disconnecting...`);
            registeredLogServers.delete(socket.id);
            socket.disconnect();
        });
    });
    socketServer.listen(Number(config_1.config.port.collector));
    console.log(`LogServer sender listening on port ${config_1.config.port.collector}`);
};
exports.setupCollectorSocketServer = setupCollectorSocketServer;
const forwardCycleData = async (data) => {
    for (const socket of registeredLogServers.values()) {
        socket.emit(exports.CycleDataWsEvent, lib_types_1.Utils.safeStringify(data));
    }
    console.log(`Forwarded cycle data to ${registeredLogServers.size} LogServers`);
};
exports.forwardCycleData = forwardCycleData;
const forwardReceiptData = async (data) => {
    for (const socket of registeredLogServers.values()) {
        socket.emit(exports.ReceiptDataWsEvent, lib_types_1.Utils.safeStringify(data));
    }
    /* prettier-ignore */ if (config_1.config.verbose)
        console.log(`Forwarded receipt data to ${registeredLogServers.size} LogServers`);
};
exports.forwardReceiptData = forwardReceiptData;
//# sourceMappingURL=CollectorSocketconnection.js.map