"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const amqp = __importStar(require("amqplib"));
class RMQConsumer {
    constructor(name, queue, prefetch = 10, callback) {
        this.conn = null;
        this.channel = null;
        this.handleConnectionClose = async () => {
            if (this.isConnClosing === true) {
                // this is triggered internally, possibly on SIGTERM/SIGINT; so no need to retry
                return Promise.resolve();
            }
            this.isConnected = false;
            return new Promise((resolve) => {
                this.retryConnection();
                resolve();
            });
        };
        this.name = name;
        this.queue = queue;
        this.prefetch = prefetch;
        this.isConnected = false;
        this.isConnClosing = false;
        this.processFn = callback;
    }
    async consume() {
        try {
            if (!this.isConnected || this.conn === null) {
                this.conn = await this.connect();
            }
            this.conn.on('error', this.handleConnectionError);
            this.conn.on('close', this.handleConnectionClose);
            this.channel = await this.conn.createChannel();
            this.channel.prefetch(this.prefetch);
            console.log(`[Consumer ${this.name}]: Started listening to queue: ${this.queue}`);
            this.channel.consume(this.queue, async (msg) => {
                if (msg) {
                    console.log(`[Consumer ${this.name}]: Received message`);
                    try {
                        const success = await this.processFn(msg.content.toString());
                        if (success === true) {
                            this.channel.ack(msg);
                            console.log(`[Consumer ${this.name}]: Successfully processed message`);
                        }
                        else {
                            this.channel.nack(msg, false, true);
                        }
                    }
                    catch (e) {
                        console.error(`Consumer [${this.name}]: Error while processing message: ${e}\nMessage: ${msg.content.toString()}`);
                        this.channel.nack(msg, false, true);
                    }
                }
            });
        }
        catch (e) {
            console.error(`[Consumer ${this.name}]: Unexpected error occurred, retyring connection. Err: ${e}`);
            throw e;
        }
    }
    async connect() {
        return await amqp.connect({
            protocol: process.env.RMQ_PROTOCOL || 'amqp',
            hostname: process.env.RMQ_HOST,
            port: process.env.RMQ_PORT ? parseInt(process.env.RMQ_PORT) : 5672,
            username: process.env.RMQ_USER,
            password: process.env.RMQ_PASS,
        });
    }
    async cleanUp() {
        this.isConnClosing = true;
        if (this.channel != null) {
            await this.channel.close();
            console.log(`[Consumer ${this.name}]: Closed channel successfully`);
        }
        if (this.conn !== null) {
            await this.conn.close();
            console.log(`[Consumer ${this.name}]: Closed connection successfully`);
        }
    }
    async handleConnectionError(error) {
        console.error(`[Consumer ${this.name}]: Connection error: ${error}`);
        this.isConnected = false;
    }
    retryConnection() {
        let attempt = 0;
        if (!this.isConnected) {
            const interval = setInterval(async () => {
                attempt++;
                console.log(`[retryConnection ${this.name}]: (Attempt ${attempt}) intitiated connection retry...`);
                try {
                    await this.consume();
                    console.log(`[retryConnection ${this.name}]: (Attempt ${attempt}) successfully connected...`);
                    this.isConnected = true;
                    clearInterval(interval);
                }
                catch (e) {
                    console.log(`[retryConnection ${this.name}]: (Attempt ${attempt}) unsuccessul. Err: ${e}`);
                }
            }, 5000); // Wait 5 seconds before retrying
        }
    }
}
exports.default = RMQConsumer;
//# sourceMappingURL=consumer.js.map