import type { FastifyError, FastifyReply, FastifyRequest, HookHandlerDoneFunction } from 'fastify';
/**
 * Usage middleware, intercepts requests and counts them by endpoint, only if usage metrics are enabled.
 */
export declare const usageMiddleware: (req: FastifyRequest, _: FastifyReply, done: HookHandlerDoneFunction) => void;
/**
 * Usage error middleware, intercepts errors and counts them by endpoint and error message, only if usage metrics are enabled.
 */
export declare const usageErrorMiddleware: (req: FastifyRequest, _: FastifyReply, error: FastifyError, done: HookHandlerDoneFunction) => void;
/**
 * Usage enable handler, enables usage metrics and resets the counters.
 */
export declare const usageEnableHandler: (req: FastifyRequest, reply: FastifyReply) => Promise<void>;
/**
 * Usage disable handler, disables usage metrics and resets the counters.
 */
export declare const usageDisableHandler: (req: FastifyRequest, reply: FastifyReply) => Promise<void>;
/**
 * Usage metrics handler, returns the usage metrics.
 */
export declare const usageMetricsHandler: (req: FastifyRequest, reply: FastifyReply) => Promise<void>;
