"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processAccountData = exports.queryTokenHolders = exports.queryTokenHolderCount = exports.queryTokenBalance = exports.queryTokensByAddress = exports.queryAccountsBetweenCycles = exports.queryAccountCountBetweenCycles = exports.queryAccountByAddress = exports.queryAccountByAccountId = exports.queryAccounts = exports.queryAccountCount = exports.bulkInsertTokens = exports.insertToken = exports.updateAccount = exports.bulkInsertAccounts = exports.insertAccount = exports.EOA_CodeHash = void 0;
const db = __importStar(require("./sqlite3storage"));
const sqlite3storage_1 = require("./sqlite3storage");
const index_1 = require("../config/index");
const types_1 = require("../types");
const util_1 = require("@ethereumjs/util");
const TxDecoder_1 = require("../class/TxDecoder");
const _1 = require(".");
const accountEntry_1 = require("./accountEntry");
const lib_types_1 = require("@shardeum-foundation/lib-types");
exports.EOA_CodeHash = '0xc5d2460186f7233c927e7db2dcc703c0e500b653ca82273b7bfad8045d85a470';
async function insertAccount(account) {
    try {
        const fields = Object.keys(account).join(', ');
        const placeholders = Object.keys(account).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValues)(account);
        const sql = 'INSERT OR REPLACE INTO accounts (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(sql, values);
        if (index_1.config.verbose)
            console.log('Successfully inserted Account', account.ethAddress || account.accountId);
        if ((0, _1.isShardeumIndexerEnabled)())
            await (0, accountEntry_1.insertAccountEntry)(account);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to insert Account or it is already stored in to database', account.accountId);
    }
}
exports.insertAccount = insertAccount;
async function bulkInsertAccounts(accounts) {
    try {
        const fields = Object.keys(accounts[0]).join(', ');
        const placeholders = Object.keys(accounts[0]).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValuesFromArray)(accounts);
        let sql = 'INSERT OR REPLACE INTO accounts (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < accounts.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(sql, values);
        console.log('Successfully bulk inserted Accounts', accounts.length);
        if ((0, _1.isShardeumIndexerEnabled)())
            await (0, accountEntry_1.bulkInsertAccountEntries)(accounts);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to bulk insert Accounts', accounts.length);
    }
}
exports.bulkInsertAccounts = bulkInsertAccounts;
async function updateAccount(_accountId, account) {
    try {
        const sql = `UPDATE accounts SET cycle = $cycle, timestamp = $timestamp, account = $account, hash = $hash WHERE accountId = $accountId `;
        await db.run(sql, {
            $cycle: account.cycle,
            $timestamp: account.timestamp,
            $account: account.account && lib_types_1.Utils.safeStringify(account.account),
            $hash: account.hash,
            $accountId: account.accountId,
        });
        if (index_1.config.verbose)
            console.log('Successfully updated Account', account.ethAddress || account.accountId);
        if ((0, _1.isShardeumIndexerEnabled)())
            await (0, accountEntry_1.updateAccountEntry)(_accountId, account);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to update Account', account);
    }
}
exports.updateAccount = updateAccount;
async function insertToken(token) {
    try {
        const fields = Object.keys(token).join(', ');
        const placeholders = Object.keys(token).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValues)(token);
        const sql = 'INSERT OR REPLACE INTO tokens (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(sql, values);
        if (index_1.config.verbose)
            console.log('Successfully inserted Token', token.ethAddress);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to insert Token or it is already stored in to database', token.ethAddress);
    }
}
exports.insertToken = insertToken;
async function bulkInsertTokens(tokens) {
    try {
        const fields = Object.keys(tokens[0]).join(', ');
        const placeholders = Object.keys(tokens[0]).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValuesFromArray)(tokens);
        let sql = 'INSERT OR REPLACE INTO tokens (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < tokens.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(sql, values);
        console.log('Successfully inserted Tokens', tokens.length);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to bulk insert Tokens', tokens.length);
    }
}
exports.bulkInsertTokens = bulkInsertTokens;
async function queryAccountCount(type) {
    let accounts = { 'COUNT(*)': 0 };
    try {
        if (type || type === types_1.AccountSearchType.All) {
            if (type === types_1.AccountSearchType.All) {
                const sql = `SELECT COUNT(*) FROM accounts`;
                accounts = await db.get(sql, []);
            }
            else if (type === types_1.AccountSearchType.CA) {
                const sql = `SELECT COUNT(*) FROM accounts WHERE accountType=? AND contractType IS NOT NULL`;
                accounts = await db.get(sql, [types_1.AccountType.Account]);
            }
            else if (type === types_1.AccountSearchType.GENERIC ||
                type === types_1.AccountSearchType.ERC_20 ||
                type === types_1.AccountSearchType.ERC_721 ||
                type === types_1.AccountSearchType.ERC_1155) {
                type =
                    type === types_1.AccountSearchType.GENERIC
                        ? types_1.ContractType.GENERIC
                        : type === types_1.AccountSearchType.ERC_20
                            ? types_1.ContractType.ERC_20
                            : type === types_1.AccountSearchType.ERC_721
                                ? types_1.ContractType.ERC_721
                                : types_1.ContractType.ERC_1155;
                const sql = `SELECT COUNT(*) FROM accounts WHERE accountType=? AND contractType=?`;
                accounts = await db.get(sql, [types_1.AccountType.Account, type]);
            }
        }
        else {
            const sql = `SELECT COUNT(*) FROM accounts WHERE accountType=?`;
            accounts = await db.get(sql, [types_1.AccountType.Account]);
        }
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('Account count', accounts);
    return accounts['COUNT(*)'] || 0;
}
exports.queryAccountCount = queryAccountCount;
async function queryAccounts(skip = 0, limit = 10, type) {
    let accounts = [];
    try {
        if (type || type === types_1.AccountSearchType.All) {
            if (type === types_1.AccountSearchType.All) {
                const sql = `SELECT * FROM accounts ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                accounts = await db.all(sql, []);
            }
            else if (type === types_1.AccountSearchType.CA) {
                const sql = `SELECT * FROM accounts WHERE accountType=? AND contractType IS NOT NULL ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                accounts = await db.all(sql, [types_1.AccountType.Account]);
            }
            else if (type === types_1.AccountSearchType.GENERIC ||
                type === types_1.AccountSearchType.ERC_20 ||
                type === types_1.AccountSearchType.ERC_721 ||
                type === types_1.AccountSearchType.ERC_1155) {
                type =
                    type === types_1.AccountSearchType.GENERIC
                        ? types_1.ContractType.GENERIC
                        : type === types_1.AccountSearchType.ERC_20
                            ? types_1.ContractType.ERC_20
                            : type === types_1.AccountSearchType.ERC_721
                                ? types_1.ContractType.ERC_721
                                : types_1.ContractType.ERC_1155;
                const sql = `SELECT * FROM accounts WHERE accountType=? AND contractType=? ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                accounts = await db.all(sql, [types_1.AccountType.Account, type]);
            }
        }
        else {
            const sql = `SELECT * FROM accounts WHERE accountType=? ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
            accounts = await db.all(sql, [types_1.AccountType.Account]);
        }
        accounts.forEach((account) => {
            if (account.account)
                account.account = lib_types_1.Utils.safeJsonParse(account.account);
            if (account.contractInfo)
                account.contractInfo = lib_types_1.Utils.safeJsonParse(account.contractInfo);
        });
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('Accounts accounts', accounts);
    return accounts;
}
exports.queryAccounts = queryAccounts;
async function queryAccountByAccountId(accountId) {
    try {
        const sql = `SELECT * FROM accounts WHERE accountId=?`;
        const account = await db.get(sql, [accountId]);
        if (account)
            account.account = lib_types_1.Utils.safeJsonParse(account.account);
        if (account && account.contractInfo)
            account.contractInfo = lib_types_1.Utils.safeJsonParse(account.contractInfo);
        if (index_1.config.verbose)
            console.log('Account accountId', account);
        return account;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryAccountByAccountId = queryAccountByAccountId;
async function queryAccountByAddress(address, accountType = types_1.AccountType.Account) {
    try {
        const sql = `SELECT * FROM accounts WHERE accountType=? AND ethAddress=? ORDER BY accountType ASC LIMIT 1`;
        const account = await db.get(sql, [accountType, address]);
        if (account)
            account.account = lib_types_1.Utils.safeJsonParse(account.account);
        if (account && account.contractInfo)
            account.contractInfo = lib_types_1.Utils.safeJsonParse(account.contractInfo);
        if (index_1.config.verbose)
            console.log('Account Address', account);
        return account;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryAccountByAddress = queryAccountByAddress;
async function queryAccountCountBetweenCycles(startCycleNumber, endCycleNumber) {
    let accounts = { 'COUNT(*)': 0 };
    try {
        const sql = `SELECT COUNT(*) FROM accounts WHERE cycle BETWEEN ? AND ?`;
        accounts = await db.get(sql, [startCycleNumber, endCycleNumber]);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose) {
        console.log('Account count between cycle', accounts);
    }
    return accounts['COUNT(*)'] || 0;
}
exports.queryAccountCountBetweenCycles = queryAccountCountBetweenCycles;
async function queryAccountsBetweenCycles(skip = 0, limit = 10000, startCycleNumber, endCycleNumber) {
    let accounts = [];
    try {
        const sql = `SELECT * FROM accounts WHERE cycle BETWEEN ? AND ? ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
        accounts = await db.all(sql, [startCycleNumber, endCycleNumber]);
        accounts.forEach((account) => {
            if (account.account)
                account.account = lib_types_1.Utils.safeJsonParse(account.account);
            if (account.contractInfo)
                account.contractInfo = lib_types_1.Utils.safeJsonParse(account.contractInfo);
        });
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose) {
        console.log('Account accounts', accounts ? accounts.length : accounts, 'skip', skip);
    }
    return accounts;
}
exports.queryAccountsBetweenCycles = queryAccountsBetweenCycles;
async function queryTokensByAddress(address, detail = false) {
    try {
        const sql = `SELECT * FROM tokens WHERE ethAddress=?`;
        const tokens = (await db.all(sql, [address]));
        const filterTokens = [];
        if (detail) {
            for (const { contractAddress, tokenValue } of tokens) {
                const accountExist = await queryAccountByAccountId(contractAddress.slice(2).toLowerCase() + '0'.repeat(24) //Search by Shardus address
                );
                if (accountExist && accountExist.contractType) {
                    filterTokens.push({
                        contractAddress: contractAddress,
                        contractInfo: accountExist.contractInfo,
                        contractType: accountExist.contractType,
                        balance: tokenValue,
                    });
                }
            }
        }
        if (index_1.config.verbose)
            console.log('Tokens of an address', tokens);
        return filterTokens;
    }
    catch (e) {
        console.log(e);
    }
    return [];
}
exports.queryTokensByAddress = queryTokensByAddress;
async function queryTokenBalance(contractAddress, addressToSearch) {
    const sql = `SELECT * FROM tokens WHERE ethAddress=? AND contractAddress=?`;
    const token = await db.get(sql, [addressToSearch, contractAddress]);
    if (index_1.config.verbose)
        console.log('Token balance', token);
    if (!token)
        return { success: false, error: 'tokenBalance is not found' };
    return {
        success: true,
        balance: token === null || token === void 0 ? void 0 : token.tokenValue,
    };
}
exports.queryTokenBalance = queryTokenBalance;
async function queryTokenHolderCount(contractAddress) {
    let tokens = { 'COUNT(*)': 0 };
    try {
        const sql = `SELECT COUNT(*) FROM tokens WHERE contractAddress=?`;
        tokens = await db.get(sql, [contractAddress]);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('Token holder count', tokens);
    return tokens['COUNT(*)'] || 0;
}
exports.queryTokenHolderCount = queryTokenHolderCount;
async function queryTokenHolders(skip = 0, limit = 10, contractAddress) {
    let tokens = [];
    try {
        const sql = `SELECT * FROM tokens WHERE contractAddress=? ORDER BY tokenValue DESC LIMIT ${limit} OFFSET ${skip}`;
        tokens = await db.all(sql, [contractAddress]);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('Token holders', tokens);
    return tokens;
}
exports.queryTokenHolders = queryTokenHolders;
async function processAccountData(accounts) {
    console.log('accounts size', accounts.length);
    if (accounts && accounts.length <= 0)
        return [];
    const bucketSize = 1000;
    let combineAccounts = [];
    const transactions = [];
    for (const account of accounts) {
        try {
            if (typeof account.data === 'string')
                account.data = lib_types_1.Utils.safeJsonParse(account.data);
        }
        catch (e) {
            console.log('Error in parsing account data', account.data);
            continue;
        }
        const accountType = account.data.accountType;
        const accObj = {
            accountId: account.accountId,
            ethAddress: account.accountId,
            cycle: account.cycleNumber,
            timestamp: account.timestamp,
            account: account.data,
            hash: account.hash,
            accountType,
            isGlobal: account.isGlobal,
        };
        if (accountType === types_1.AccountType.Account ||
            accountType === types_1.AccountType.ContractStorage ||
            accountType === types_1.AccountType.ContractCode) {
            accObj.ethAddress = account.data.ethAddress.toLowerCase();
            if (index_1.config.processData.decodeContractInfo &&
                accountType === types_1.AccountType.Account &&
                'account' in accObj.account &&
                (0, util_1.bytesToHex)(Uint8Array.from(Object.values(accObj.account.account.codeHash))) !== exports.EOA_CodeHash) {
                const { contractInfo, contractType } = await (0, TxDecoder_1.getContractInfo)(accObj.ethAddress);
                accObj.contractInfo = contractInfo;
                accObj.contractType = contractType;
                await insertAccount(accObj);
                await (0, accountEntry_1.insertAccountEntry)(accObj);
                continue;
            }
        }
        else if (accountType === types_1.AccountType.NetworkAccount ||
            accountType === types_1.AccountType.DevAccount ||
            accountType === types_1.AccountType.NodeAccount ||
            accountType === types_1.AccountType.NodeAccount2 ||
            accountType === types_1.AccountType.SecureAccount) {
            accObj.ethAddress = account.accountId; // Adding accountId as ethAddess for these account types for now; since we need ethAddress for mysql index
        }
        combineAccounts.push(accObj);
        if (accountType === types_1.AccountType.Receipt ||
            accountType === types_1.AccountType.NodeRewardReceipt ||
            accountType === types_1.AccountType.StakeReceipt ||
            accountType === types_1.AccountType.UnstakeReceipt) {
            transactions.push(account);
        }
        if (combineAccounts.length >= bucketSize) {
            await bulkInsertAccounts(combineAccounts);
            combineAccounts = [];
        }
    }
    if (combineAccounts.length > 0)
        await bulkInsertAccounts(combineAccounts);
    return transactions;
}
exports.processAccountData = processAccountData;
//# sourceMappingURL=account.js.map