"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryAccountEntryCount = exports.updateAccountEntry = exports.bulkInsertAccountEntries = exports.insertAccountEntry = void 0;
const index_1 = require("../config/index");
const db = __importStar(require("./sqlite3storage"));
const lib_types_1 = require("@shardeum-foundation/lib-types");
async function insertAccountEntry(account) {
    try {
        const accountEntry = {
            accountId: account.accountId,
            timestamp: account.timestamp,
            data: account.account,
        };
        const fields = Object.keys(accountEntry).join(', ');
        const placeholders = Object.keys(accountEntry).fill('?').join(', ');
        const values = db.extractValues(accountEntry);
        const sql = 'INSERT OR REPLACE INTO accountsEntry (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(sql, values, 'shardeumIndexer');
        if (index_1.config.verbose)
            console.log('ShardeumIndexer: Successfully inserted AccountEntry', account.ethAddress || account.accountId);
    }
    catch (e) {
        console.log(e);
        console.log('ShardeumIndexer: Unable to insert AccountEntry or it is already stored in to database', account.accountId);
    }
}
exports.insertAccountEntry = insertAccountEntry;
async function bulkInsertAccountEntries(accounts) {
    try {
        const accountEntries = [];
        for (const account of accounts) {
            const accountEntry = {
                accountId: account.accountId,
                timestamp: account.timestamp,
                data: account.account,
            };
            accountEntries.push(accountEntry);
        }
        const fields = Object.keys(accountEntries[0]).join(', ');
        const placeholders = Object.keys(accountEntries[0]).fill('?').join(', ');
        const values = db.extractValuesFromArray(accountEntries);
        let sql = 'INSERT OR REPLACE INTO accountsEntry (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < accountEntries.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(sql, values, 'shardeumIndexer');
        console.log('ShardeumIndexer: Successfully bulk inserted AccountEntries', accountEntries.length);
    }
    catch (e) {
        console.log(e);
        console.log('ShardeumIndexer: Unable to bulk insert AccountEntries', accounts.length);
    }
}
exports.bulkInsertAccountEntries = bulkInsertAccountEntries;
async function updateAccountEntry(_accountId, account) {
    try {
        const sql = `UPDATE accountsEntry SET timestamp = $timestamp, data = $account WHERE accountId = $accountId `;
        await db.run(sql, {
            $timestamp: account.timestamp,
            $account: account.account && lib_types_1.Utils.safeStringify(account.account),
            $accountId: account.accountId,
        }, 'shardeumIndexer');
        if (index_1.config.verbose)
            console.log('ShardeumIndexer: Successfully updated AccountEntry', account.ethAddress || account.accountId);
    }
    catch (e) {
        console.log(e);
        console.log('ShardeumIndexer: Unable to update AccountEntry', account);
    }
}
exports.updateAccountEntry = updateAccountEntry;
async function queryAccountEntryCount() {
    let accountEntries = { 'COUNT(*)': 0 };
    try {
        const sql = `SELECT COUNT(*) FROM accountsEntry`;
        accountEntries = await db.get(sql, [], 'shardeumIndexer');
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('AccountEntry count', accountEntries);
    return accountEntries['COUNT(*)'] || 0;
}
exports.queryAccountEntryCount = queryAccountEntryCount;
//# sourceMappingURL=accountEntry.js.map