"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryAccountHistoryStateCount = exports.queryAccountHistoryState = exports.bulkInsertAccountHistoryStates = exports.insertAccountHistoryState = void 0;
const db = __importStar(require("./sqlite3storage"));
const sqlite3storage_1 = require("./sqlite3storage");
const index_1 = require("../config/index");
const types_1 = require("../types");
const ReceiptDB = __importStar(require("./receipt"));
async function insertAccountHistoryState(accountHistoryState) {
    try {
        const fields = Object.keys(accountHistoryState).join(', ');
        const placeholders = Object.keys(accountHistoryState).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValues)(accountHistoryState);
        const sql = 'INSERT OR REPLACE INTO accountHistoryState (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(sql, values);
        if (index_1.config.verbose)
            console.log('Successfully inserted AccountHistoryState', accountHistoryState.accountId, accountHistoryState.receiptId);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to insert AccountHistoryState or it is already stored in to database', accountHistoryState.accountId, accountHistoryState.receiptId);
    }
}
exports.insertAccountHistoryState = insertAccountHistoryState;
async function bulkInsertAccountHistoryStates(accountHistoryStates) {
    try {
        const fields = Object.keys(accountHistoryStates[0]).join(', ');
        const placeholders = Object.keys(accountHistoryStates[0]).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValuesFromArray)(accountHistoryStates);
        let sql = 'INSERT OR REPLACE INTO accountHistoryState (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < accountHistoryStates.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(sql, values);
        console.log('Successfully bulk inserted AccountHistoryStates', accountHistoryStates.length);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to bulk insert AccountHistoryStates', accountHistoryStates.length);
    }
}
exports.bulkInsertAccountHistoryStates = bulkInsertAccountHistoryStates;
async function queryAccountHistoryState(_accountId, blockNumber = undefined, blockHash = undefined) {
    try {
        let sql = `SELECT * FROM accountHistoryState WHERE accountId=? AND `;
        const values = [_accountId];
        if (blockNumber) {
            sql += `blockNumber<? ORDER BY blockNumber DESC LIMIT 1`;
            values.push(blockNumber);
        }
        // if (blockHash) {
        //   sql += `blockHash=? DESC LIMIT 1`
        //   values.push(blockHash)
        // }
        const accountHistoryState = await db.get(sql, values);
        if (accountHistoryState) {
            if (index_1.config.verbose)
                console.log('AccountHistoryState', accountHistoryState);
            const receipt = await ReceiptDB.queryReceiptByReceiptId(accountHistoryState.receiptId);
            if (!receipt) {
                console.log('Unable to find receipt for AccountHistoryState', accountHistoryState.receiptId);
                return null;
            }
            const filterAccount = receipt.afterStates.filter((account) => account.accountId === _accountId);
            if (filterAccount.length === 0) {
                console.log('Unable to find account in receipt for AccountHistoryState', accountHistoryState.receiptId);
                return null;
            }
            const account = filterAccount[0];
            const accountType = account.data.accountType;
            let ethAddress;
            if (accountType === types_1.AccountType.Account ||
                accountType === types_1.AccountType.ContractStorage ||
                accountType === types_1.AccountType.ContractCode)
                ethAddress = account.data.ethAddress;
            else
                ethAddress = account.accountId;
            const accObj = {
                accountId: account.accountId,
                cycle: receipt.cycle,
                timestamp: account.timestamp,
                account: account.data,
                hash: account.hash,
                accountType,
                isGlobal: account.isGlobal,
                ethAddress,
            };
            return accObj;
        }
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryAccountHistoryState = queryAccountHistoryState;
async function queryAccountHistoryStateCount() {
    let accountHistoryStates = { 'COUNT(*)': 0 };
    try {
        const sql = `SELECT COUNT(*) FROM accountHistoryState`;
        accountHistoryStates = await db.get(sql, []);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('AccountHistoryState count', accountHistoryStates);
    return accountHistoryStates['COUNT(*)'] || 0;
}
exports.queryAccountHistoryStateCount = queryAccountHistoryStateCount;
//# sourceMappingURL=accountHistoryState.js.map