"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryCycleCount = exports.queryCycleByCounter = exports.queryCycleByMarker = exports.queryCycleRecordsBetween = exports.queryLatestCycleRecords = exports.insertOrUpdateCycle = exports.updateCycle = exports.bulkInsertCycles = exports.insertCycle = exports.isCycle = exports.Collection = void 0;
const db = __importStar(require("./sqlite3storage"));
const sqlite3storage_1 = require("./sqlite3storage");
const index_1 = require("../config/index");
const _1 = require(".");
const block_1 = require("./block");
const receipt_1 = require("./receipt");
const originalTxData_1 = require("./originalTxData");
const lib_types_1 = require("@shardeum-foundation/lib-types");
function isCycle(obj) {
    return obj.cycleRecord !== undefined && obj.cycleMarker !== undefined;
}
exports.isCycle = isCycle;
async function insertCycle(cycle) {
    try {
        const fields = Object.keys(cycle).join(', ');
        const placeholders = Object.keys(cycle).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValues)(cycle);
        const sql = 'INSERT OR REPLACE INTO cycles (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(sql, values);
        if (index_1.config.verbose)
            console.log('Successfully inserted Cycle', cycle.cycleRecord.counter, cycle.cycleMarker);
        if ((0, _1.isBlockIndexingEnabled)())
            await (0, block_1.upsertBlocksForCycle)(cycle);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to insert cycle or it is already stored in to database', cycle.cycleRecord.counter, cycle.cycleMarker);
    }
}
exports.insertCycle = insertCycle;
async function bulkInsertCycles(cycles) {
    try {
        const fields = Object.keys(cycles[0]).join(', ');
        const placeholders = Object.keys(cycles[0]).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValuesFromArray)(cycles);
        let sql = 'INSERT OR REPLACE INTO cycles (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < cycles.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(sql, values);
        console.log('Successfully bulk inserted Cycles', cycles.length);
        if ((0, _1.isBlockIndexingEnabled)())
            await (0, block_1.upsertBlocksForCycles)(cycles);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to bulk insert Cycles', cycles.length);
    }
}
exports.bulkInsertCycles = bulkInsertCycles;
async function updateCycle(marker, cycle) {
    try {
        const sql = `UPDATE cycles SET counter = $counter, cycleRecord = $cycleRecord WHERE cycleMarker = $marker `;
        await db.run(sql, {
            $counter: cycle.counter,
            $cycleRecord: cycle.cycleRecord && lib_types_1.Utils.safeStringify(cycle.cycleRecord),
            $marker: marker,
        });
        if (index_1.config.verbose)
            console.log('Updated cycle for counter', cycle.cycleRecord.counter, cycle.cycleMarker);
        if ((0, _1.isBlockIndexingEnabled)())
            await (0, block_1.upsertBlocksForCycle)(cycle);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to update Cycle', cycle.cycleMarker);
    }
}
exports.updateCycle = updateCycle;
async function insertOrUpdateCycle(cycle) {
    if (cycle && cycle.cycleRecord && cycle.cycleMarker) {
        const cycleInfo = {
            counter: cycle.cycleRecord.counter,
            cycleRecord: cycle.cycleRecord,
            cycleMarker: cycle.cycleMarker,
        };
        const cycleExist = await queryCycleByMarker(cycle.cycleMarker);
        if (index_1.config.verbose)
            console.log('cycleExist', cycleExist);
        if (cycleExist) {
            if (lib_types_1.Utils.safeStringify(cycleInfo) !== lib_types_1.Utils.safeStringify(cycleExist))
                await updateCycle(cycleInfo.cycleMarker, cycleInfo);
        }
        else {
            await insertCycle(cycleInfo);
            // Clean up receipts map that are older than 5 minutes
            const CLEAN_UP_TIMESTMAP_MS = Date.now() - 5 * 60 * 1000;
            (0, receipt_1.cleanOldReceiptsMap)(CLEAN_UP_TIMESTMAP_MS);
            (0, originalTxData_1.cleanOldOriginalTxsMap)(CLEAN_UP_TIMESTMAP_MS);
        }
    }
    else {
        console.log('No cycleRecord or cycleMarker in cycle,', cycle);
    }
}
exports.insertOrUpdateCycle = insertOrUpdateCycle;
async function queryLatestCycleRecords(count) {
    try {
        const sql = `SELECT * FROM cycles ORDER BY counter DESC LIMIT ${count}`;
        const cycleRecords = await db.all(sql);
        if (cycleRecords.length > 0) {
            cycleRecords.forEach((cycleRecord) => {
                if (cycleRecord.cycleRecord)
                    cycleRecord.cycleRecord = lib_types_1.Utils.safeJsonParse(cycleRecord.cycleRecord);
            });
        }
        if (index_1.config.verbose)
            console.log('cycle latest', cycleRecords);
        return cycleRecords;
    }
    catch (e) {
        console.log(e);
    }
    return [];
}
exports.queryLatestCycleRecords = queryLatestCycleRecords;
async function queryCycleRecordsBetween(start, end) {
    try {
        const sql = `SELECT * FROM cycles WHERE counter BETWEEN ? AND ? ORDER BY counter DESC`;
        const cycles = await db.all(sql, [start, end]);
        if (cycles.length > 0) {
            cycles.forEach((cycleRecord) => {
                if (cycleRecord.cycleRecord)
                    cycleRecord.cycleRecord = lib_types_1.Utils.safeJsonParse(cycleRecord.cycleRecord);
            });
        }
        if (index_1.config.verbose)
            console.log('cycle between', cycles);
        return cycles;
    }
    catch (e) {
        console.log(e);
    }
    return [];
}
exports.queryCycleRecordsBetween = queryCycleRecordsBetween;
async function queryCycleByMarker(marker) {
    try {
        const sql = `SELECT * FROM cycles WHERE cycleMarker=? LIMIT 1`;
        const cycleRecord = await db.get(sql, [marker]);
        if (cycleRecord) {
            if (cycleRecord.cycleRecord)
                cycleRecord.cycleRecord = lib_types_1.Utils.safeJsonParse(cycleRecord.cycleRecord);
        }
        if (index_1.config.verbose)
            console.log('cycle marker', cycleRecord);
        return cycleRecord;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryCycleByMarker = queryCycleByMarker;
async function queryCycleByCounter(counter) {
    try {
        const sql = `SELECT * FROM cycles WHERE counter=? LIMIT 1`;
        const cycleRecord = await db.get(sql, [counter]);
        if (cycleRecord) {
            if (cycleRecord.cycleRecord)
                cycleRecord.cycleRecord = lib_types_1.Utils.safeJsonParse(cycleRecord.cycleRecord);
        }
        if (index_1.config.verbose)
            console.log('cycle counter', cycleRecord);
        return cycleRecord;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryCycleByCounter = queryCycleByCounter;
async function queryCycleCount() {
    let cycles = { 'COUNT(*)': 0 };
    try {
        const sql = `SELECT COUNT(*) FROM cycles`;
        cycles = await db.get(sql, []);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('Cycle count', cycles);
    return cycles['COUNT(*)'] || 0;
}
exports.queryCycleCount = queryCycleCount;
//# sourceMappingURL=cycle.js.map