"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryLogsByFilter = exports.queryLogsBetweenCycles = exports.queryLogCountBetweenCycles = exports.queryLogs = exports.queryLogCount = exports.bulkInsertLogs = exports.insertLog = void 0;
/* eslint-disable no-empty */
const db = __importStar(require("./sqlite3storage"));
const sqlite3storage_1 = require("./sqlite3storage");
const index_1 = require("../config/index");
const lodash_1 = require("lodash");
const lib_types_1 = require("@shardeum-foundation/lib-types");
async function insertLog(log) {
    try {
        const fields = Object.keys(log).join(', ');
        const placeholders = Object.keys(log).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValues)(log);
        const sql = 'INSERT OR REPLACE INTO logs (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(sql, values);
        if (index_1.config.verbose)
            console.log('Successfully inserted Log', log.txHash, log.contractAddress);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to insert Log or it is already stored in to database', log.txHash, log.contractAddress);
    }
}
exports.insertLog = insertLog;
async function bulkInsertLogs(logs) {
    try {
        const fields = Object.keys(logs[0]).join(', ');
        const placeholders = Object.keys(logs[0]).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValuesFromArray)(logs);
        let sql = 'INSERT OR REPLACE INTO logs (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < logs.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(sql, values);
        console.log('Successfully bulk inserted Logs', logs.length);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to bulk insert Logs', logs.length);
    }
}
exports.bulkInsertLogs = bulkInsertLogs;
function buildLogQueryString(request, countOnly, type) {
    let sql;
    const queryParams = [];
    const values = [];
    if (countOnly) {
        sql = 'SELECT COUNT(txHash) FROM logs ';
        if (type === 'txs')
            sql = 'SELECT COUNT(DISTINCT(txHash)) FROM logs ';
    }
    else {
        sql = 'SELECT * FROM logs ';
    }
    const fromBlock = request.fromBlock;
    const toBlock = request.toBlock;
    if (fromBlock && toBlock) {
        queryParams.push(`blockNumber BETWEEN ? AND ?`);
        values.push(fromBlock, toBlock);
    }
    else if (request.blockHash) {
        queryParams.push(`blockHash=?`);
        values.push(request.blockHash);
    }
    if (request.address) {
        queryParams.push(`contractAddress=?`);
        values.push(request.address);
    }
    const createTopicQuery = (topicIndex, topicValue) => {
        const hexPattern = /^0x[a-fA-F0-9]{64}$/;
        if (Array.isArray(topicValue)) {
            const validHexValues = topicValue.filter((value) => typeof value === 'string' && hexPattern.test(value));
            if (validHexValues.length > 0) {
                const query = `topic${topicIndex} IN (${validHexValues.map(() => '?').join(',')})`;
                queryParams.push(query);
                values.push(...validHexValues);
            }
        }
        else if (typeof topicValue === 'string' && hexPattern.test(topicValue)) {
            queryParams.push(`topic${topicIndex}=?`);
            values.push(topicValue);
        }
    };
    // Handling topics array
    if (Array.isArray(request.topics)) {
        request.topics.forEach((topic, index) => createTopicQuery(index, topic));
    }
    sql = `${sql}${queryParams.length > 0 ? ` WHERE ${queryParams.join(' AND ')}` : ''}`;
    return { sql, values };
}
async function queryLogCount(contractAddress, topics, fromBlock, toBlock, blockHash, type = undefined) {
    let logs = { 'COUNT(txHash)': 0 };
    try {
        const { sql, values: inputs } = buildLogQueryString({
            address: contractAddress,
            topics,
            fromBlock,
            toBlock,
            blockHash,
        }, true, type);
        if (index_1.config.verbose)
            console.log(sql, inputs);
        logs = await db.get(sql, inputs);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('Log count', logs);
    if (logs && type === 'txs')
        return logs['COUNT(DISTINCT(txHash))'];
    else if (logs)
        return logs['COUNT(txHash)'];
    else
        return 0;
}
exports.queryLogCount = queryLogCount;
async function queryLogs(skip = 0, limit = 10, contractAddress, topics, fromBlock, toBlock, type) {
    let logs = [];
    try {
        const { sql, values: inputs } = buildLogQueryString({
            address: contractAddress,
            topics,
            fromBlock,
            toBlock,
        }, false, type);
        let sqlQueryExtension = ` ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
        if (type === 'txs') {
            sqlQueryExtension = ` GROUP BY txHash` + sqlQueryExtension;
        }
        if (index_1.config.verbose)
            console.log(sql, inputs);
        logs = await db.all(sql + sqlQueryExtension, inputs);
        if (logs.length > 0) {
            logs.forEach((log) => {
                if (log.log)
                    log.log = lib_types_1.Utils.safeJsonParse(log.log);
            });
        }
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('Log logs', logs);
    return logs;
}
exports.queryLogs = queryLogs;
async function queryLogCountBetweenCycles(startCycleNumber, endCycleNumber) {
    let logs = { 'COUNT(*)': 0 };
    try {
        const sql = `SELECT COUNT(*) FROM logs WHERE cycle BETWEEN ? AND ?`;
        logs = await db.get(sql, [startCycleNumber, endCycleNumber]);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose) {
        console.log('Log count between cycle', logs);
    }
    return logs['COUNT(*)'] || 0;
}
exports.queryLogCountBetweenCycles = queryLogCountBetweenCycles;
async function queryLogsBetweenCycles(skip = 0, limit = 10000, startCycleNumber, endCycleNumber) {
    let logs = [];
    try {
        const sql = `SELECT * FROM logs WHERE cycle BETWEEN ? AND ? ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
        logs = await db.all(sql, [startCycleNumber, endCycleNumber]);
        if (logs.length > 0) {
            logs.forEach((log) => {
                if (log.log)
                    log.log = lib_types_1.Utils.safeJsonParse(log.log);
            });
        }
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose) {
        console.log('Log logs between cycles', logs ? logs.length : logs, 'skip', skip);
    }
    return logs;
}
exports.queryLogsBetweenCycles = queryLogsBetweenCycles;
async function queryLogsByFilter(logFilter, limit = 5000) {
    let logs = [];
    const queryParams = [];
    function createSqlFromEvmLogFilter(filter) {
        const { fromBlock, toBlock, address, topics, blockHash } = filter;
        let sql = `SELECT log FROM logs WHERE 1 = 1`;
        if ((0, lodash_1.isArray)(address) && address.length > 0) {
            sql += ` AND contractAddress IN (${address.map(() => `?`).join(',')})`;
            for (const addr of address) {
                queryParams.push(addr.toLowerCase());
            }
        }
        if (blockHash) {
            sql += ` AND blockHash = ?`;
            queryParams.push(blockHash.toLowerCase());
        }
        else {
            if (fromBlock == 'latest') {
                sql += ` AND blockNumber >= (
                        SELECT MAX(blockNumber)
                        FROM logs
                  )`;
            }
            if (fromBlock == 'earliest') {
                // genesis block
                sql += ` AND blockNumber >= 0`;
            }
            if (fromBlock && fromBlock !== 'latest' && fromBlock !== 'earliest') {
                sql += ` AND blockNumber >= ?`;
                queryParams.push(Number(fromBlock));
            }
            if (toBlock == 'latest') {
                sql += ` AND blockNumber <= (
                        SELECT MAX(blockNumber)
                        FROM logs
                  )`;
            }
            if (toBlock == 'earliest') {
                // genesis block
                sql += ` AND blockNumber <= 0`;
            }
            if (toBlock && toBlock !== 'latest' && toBlock !== 'earliest') {
                sql += ` AND blockNumber <= ?`;
                queryParams.push(Number(toBlock));
            }
        }
        if (topics[0]) {
            sql += ` AND topic0 = ?`;
            queryParams.push(topics[0].toLowerCase());
        }
        if (topics[1]) {
            sql += ` AND topic1 = ?`;
            queryParams.push(topics[1].toLowerCase());
        }
        if (topics[2]) {
            sql += ` AND topic2 = ?`;
            queryParams.push(topics[2].toLowerCase());
        }
        if (topics[3]) {
            sql += ` AND topic3 = ?`;
            queryParams.push(topics[3].toLowerCase());
        }
        sql += ` ORDER BY blockNumber ASC LIMIT ?;`;
        queryParams.push(limit);
        if (index_1.config.verbose)
            console.log(`queryLogsByFilter: Query: `, sql, queryParams);
        return sql;
    }
    const sql = createSqlFromEvmLogFilter(logFilter);
    logs = await db.all(sql, queryParams);
    if (logs.length > 0) {
        logs.forEach((log) => {
            if (log.log)
                log.log = lib_types_1.Utils.safeJsonParse(log.log);
        });
    }
    return logs;
}
exports.queryLogsByFilter = queryLogsByFilter;
//# sourceMappingURL=log.js.map