"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanOldOriginalTxsMap = exports.queryOriginalTxDataCountByCycles = exports.queryOriginalTxDataByTxHash = exports.queryOriginalTxDataByTxId = exports.queryOriginalTxsData = exports.queryOriginalTxDataCount = exports.processOriginalTxData = exports.bulkInsertOriginalTxsData = exports.insertOriginalTxData = exports.originalTxsMap = void 0;
const db = __importStar(require("./sqlite3storage"));
const sqlite3storage_1 = require("./sqlite3storage");
const index_1 = require("../config/index");
const types_1 = require("../types");
const decodeEVMRawTx_1 = require("../utils/decodeEVMRawTx");
const util_1 = require("@ethereumjs/util");
const lib_types_1 = require("@shardeum-foundation/lib-types");
var OriginalTxDataType;
(function (OriginalTxDataType) {
    OriginalTxDataType["OriginalTxData"] = "originalTxsData";
    OriginalTxDataType["OriginalTxData2"] = "originalTxsData2";
})(OriginalTxDataType || (OriginalTxDataType = {}));
exports.originalTxsMap = new Map();
async function insertOriginalTxData(originalTxData, tableName) {
    try {
        const fields = Object.keys(originalTxData).join(', ');
        const placeholders = Object.keys(originalTxData).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValues)(originalTxData);
        const sql = `INSERT OR REPLACE INTO ${tableName} (` + fields + ') VALUES (' + placeholders + ')';
        await db.run(sql, values);
        if (index_1.config.verbose)
            console.log(`Successfully inserted ${tableName}`, originalTxData.txId);
    }
    catch (e) {
        console.log(e);
        console.log(`Unable to insert ${tableName} or it is already stored in to database`, originalTxData);
    }
}
exports.insertOriginalTxData = insertOriginalTxData;
async function bulkInsertOriginalTxsData(originalTxsData, tableName) {
    try {
        const fields = Object.keys(originalTxsData[0]).join(', ');
        const placeholders = Object.keys(originalTxsData[0]).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValuesFromArray)(originalTxsData);
        let sql = `INSERT OR REPLACE INTO ${tableName} (` + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < originalTxsData.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(sql, values);
        console.log(`Successfully bulk inserted ${tableName}`, originalTxsData.length);
    }
    catch (e) {
        console.log(e);
        console.log(`Unable to bulk insert ${tableName}`, originalTxsData.length);
        throw e; // check with Achal/Jai
    }
}
exports.bulkInsertOriginalTxsData = bulkInsertOriginalTxsData;
async function processOriginalTxData(originalTxsData, saveOnlyNewData = false) {
    if (originalTxsData && originalTxsData.length <= 0)
        return;
    const bucketSize = 1000;
    let combineOriginalTxsData = [];
    let combineOriginalTxsData2 = [];
    for (const originalTxData of originalTxsData) {
        const { txId, timestamp } = originalTxData;
        if (exports.originalTxsMap.has(txId) && exports.originalTxsMap.get(txId) === timestamp)
            continue;
        exports.originalTxsMap.set(txId, timestamp);
        /* prettier-ignore */ if (index_1.config.verbose)
            console.log('originalTxData', originalTxData);
        if (saveOnlyNewData) {
            const originalTxDataExist = await queryOriginalTxDataByTxId(txId);
            if (originalTxDataExist)
                continue;
        }
        combineOriginalTxsData.push(originalTxData);
        if (combineOriginalTxsData.length >= bucketSize) {
            await bulkInsertOriginalTxsData(combineOriginalTxsData, OriginalTxDataType.OriginalTxData);
            combineOriginalTxsData = [];
        }
        if (!index_1.config.processData.indexOriginalTxData)
            continue;
        try {
            if (originalTxData.originalTxData.tx.raw) {
                // EVM Tx
                const txObj = (0, decodeEVMRawTx_1.getTransactionObj)(originalTxData.originalTxData.tx);
                /* prettier-ignore */ if (index_1.config.verbose)
                    console.log('txObj', txObj);
                if (txObj) {
                    let transactionType = types_1.TransactionType.Receipt;
                    if ((0, decodeEVMRawTx_1.isStakingEVMTx)(txObj)) {
                        const internalTxData = (0, decodeEVMRawTx_1.getStakeTxBlobFromEVMTx)(txObj);
                        /* prettier-ignore */ if (index_1.config.verbose)
                            console.log('internalTxData', internalTxData);
                        if (internalTxData) {
                            if (internalTxData.internalTXType === types_1.InternalTXType.Stake) {
                                transactionType = types_1.TransactionType.StakeReceipt;
                            }
                            else if (internalTxData.internalTXType === types_1.InternalTXType.Unstake) {
                                transactionType = types_1.TransactionType.UnstakeReceipt;
                            }
                            else
                                console.log('Unknown staking evm tx type', internalTxData);
                        }
                    }
                    combineOriginalTxsData2.push({
                        txId: originalTxData.txId,
                        timestamp: originalTxData.timestamp,
                        cycle: originalTxData.cycle,
                        txHash: (0, util_1.bytesToHex)(txObj.hash()),
                        transactionType,
                    });
                }
                else {
                    console.log('Unable to get txObj from EVM raw tx', originalTxData.originalTxData.tx.raw);
                }
            }
            else {
                combineOriginalTxsData2.push({
                    txId: originalTxData.txId,
                    timestamp: originalTxData.timestamp,
                    cycle: originalTxData.cycle,
                    txHash: '0x' + originalTxData.txId,
                    transactionType: types_1.TransactionType.InternalTxReceipt,
                });
            }
        }
        catch (e) {
            console.log('Error in processing original Tx data', originalTxData.txId, e);
        }
        if (combineOriginalTxsData2.length >= bucketSize) {
            await bulkInsertOriginalTxsData(combineOriginalTxsData2, OriginalTxDataType.OriginalTxData2);
            combineOriginalTxsData2 = [];
        }
    }
    if (combineOriginalTxsData.length > 0)
        await bulkInsertOriginalTxsData(combineOriginalTxsData, OriginalTxDataType.OriginalTxData);
    if (combineOriginalTxsData2.length > 0)
        await bulkInsertOriginalTxsData(combineOriginalTxsData2, OriginalTxDataType.OriginalTxData2);
}
exports.processOriginalTxData = processOriginalTxData;
async function queryOriginalTxDataCount(txType, afterTimestamp, startCycle, endCycle) {
    let originalTxsData = { 'COUNT(*)': 0 };
    try {
        let sql = `SELECT COUNT(*) FROM originalTxsData`;
        const values = [];
        if (startCycle && endCycle) {
            sql += ` WHERE cycle BETWEEN ? AND ?`;
            values.push(startCycle, endCycle);
        }
        if (afterTimestamp) {
            if (startCycle && endCycle)
                sql += ` AND timestamp>?`;
            else
                sql += ` WHERE timestamp>?`;
            values.push(afterTimestamp);
        }
        if (txType) {
            sql = sql.replace('originalTxsData', 'originalTxsData2');
            if ((startCycle && endCycle) || afterTimestamp)
                sql += ` AND`;
            else
                sql += ` WHERE`;
            if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                sql += ` transactionType!=?`;
                values.push(types_1.TransactionType.InternalTxReceipt);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                sql += ` transactionType=?`;
                values.push(ty);
            }
        }
        originalTxsData = await db.get(sql, values);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('OriginalTxData count', originalTxsData);
    return originalTxsData['COUNT(*)'] || 0;
}
exports.queryOriginalTxDataCount = queryOriginalTxDataCount;
async function queryOriginalTxsData(skip = 0, limit = 10, txType, afterTimestamp, startCycle, endCycle) {
    let originalTxsData = [];
    try {
        let sql = `SELECT * FROM originalTxsData`;
        const sqlSuffix = ` ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
        const values = [];
        if (startCycle && endCycle) {
            sql += ` WHERE cycle BETWEEN ? AND ?`;
            values.push(startCycle, endCycle);
        }
        if (afterTimestamp) {
            if (startCycle && endCycle)
                sql += ` AND timestamp>?`;
            else
                sql += ` WHERE timestamp>?`;
            values.push(afterTimestamp);
        }
        if (txType) {
            sql = sql.replace('originalTxsData', 'originalTxsData2');
            if ((startCycle && endCycle) || afterTimestamp)
                sql += ` AND`;
            else
                sql += ` WHERE`;
            if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                sql += ` transactionType!=?`;
                values.push(types_1.TransactionType.InternalTxReceipt);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                sql += ` transactionType=?`;
                values.push(ty);
            }
        }
        sql += sqlSuffix;
        originalTxsData = await db.all(sql, values);
        for (const originalTxData of originalTxsData) {
            if (txType) {
                const sql = `SELECT * FROM originalTxsData WHERE txId=?`;
                const originalTxDataById = await db.get(sql, [originalTxData.txId]);
                originalTxData.originalTxData = originalTxDataById.originalTxData;
                originalTxData.sign = originalTxDataById.sign;
            }
            if (originalTxData.originalTxData)
                originalTxData.originalTxData = lib_types_1.Utils.safeJsonParse(originalTxData.originalTxData);
            if (originalTxData.sign)
                originalTxData.sign = lib_types_1.Utils.safeJsonParse(originalTxData.sign);
        }
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('OriginalTxData originalTxsData', originalTxsData);
    return originalTxsData;
}
exports.queryOriginalTxsData = queryOriginalTxsData;
async function queryOriginalTxDataByTxId(txId) {
    try {
        const sql = `SELECT * FROM originalTxsData WHERE txId=?`;
        const originalTxData = await db.get(sql, [txId]);
        if (originalTxData) {
            if (originalTxData.originalTxData)
                originalTxData.originalTxData = lib_types_1.Utils.safeJsonParse(originalTxData.originalTxData);
            if (originalTxData.sign)
                originalTxData.sign = lib_types_1.Utils.safeJsonParse(originalTxData.sign);
        }
        if (index_1.config.verbose)
            console.log('OriginalTxData txId', originalTxData);
        return originalTxData;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryOriginalTxDataByTxId = queryOriginalTxDataByTxId;
async function queryOriginalTxDataByTxHash(txHash) {
    try {
        const sql = `SELECT * FROM originalTxsData2 WHERE txHash=?`;
        const originalTxData = await db.get(sql, [txHash]);
        if (originalTxData) {
            const sql = `SELECT * FROM originalTxsData WHERE txId=?`;
            const originalTxDataById = await db.get(sql, [originalTxData.txId]);
            originalTxData.originalTxData = originalTxDataById.originalTxData;
            originalTxData.sign = originalTxDataById.sign;
            if (originalTxData.originalTxData)
                originalTxData.originalTxData = lib_types_1.Utils.safeJsonParse(originalTxData.originalTxData);
            if (originalTxData.sign)
                originalTxData.sign = lib_types_1.Utils.safeJsonParse(originalTxData.sign);
        }
        if (index_1.config.verbose)
            console.log('OriginalTxData txHash', originalTxData);
        return originalTxData;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryOriginalTxDataByTxHash = queryOriginalTxDataByTxHash;
async function queryOriginalTxDataCountByCycles(start, end) {
    let originalTxsData = [];
    try {
        const sql = `SELECT cycle, COUNT(*) FROM originalTxsData GROUP BY cycle HAVING cycle BETWEEN ? AND ? ORDER BY cycle ASC`;
        originalTxsData = await db.all(sql, [start, end]);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('OriginalTxData count by cycles', originalTxsData);
    return originalTxsData.map((originalTxData) => {
        return {
            originalTxsData: originalTxData['COUNT(*)'],
            cycle: originalTxData.cycle,
        };
    });
}
exports.queryOriginalTxDataCountByCycles = queryOriginalTxDataCountByCycles;
function cleanOldOriginalTxsMap(timestamp) {
    for (const [key, value] of exports.originalTxsMap) {
        if (value < timestamp) {
            exports.originalTxsMap.delete(key);
        }
    }
    if (index_1.config.verbose)
        console.log('Clean Old OriginalTxs map!', timestamp, exports.originalTxsMap);
}
exports.cleanOldOriginalTxsMap = cleanOldOriginalTxsMap;
//# sourceMappingURL=originalTxData.js.map