"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanOldReceiptsMap = exports.queryReceiptCountBetweenCycles = exports.queryReceiptsBetweenCycles = exports.queryReceiptCountByCycles = exports.queryReceiptCount = exports.queryReceipts = exports.queryLatestReceipts = exports.queryReceiptByReceiptId = exports.processReceiptData = exports.bulkInsertReceipts = exports.insertReceipt = exports.receiptsMap = void 0;
const config_1 = require("../config");
const AccountDB = __importStar(require("./account"));
const TransactionDB = __importStar(require("./transaction"));
const AccountHistoryStateDB = __importStar(require("./accountHistoryState"));
const lib_types_1 = require("@shardeum-foundation/lib-types");
const types_1 = require("../types");
const db = __importStar(require("./sqlite3storage"));
const sqlite3storage_1 = require("./sqlite3storage");
const TxDecoder_1 = require("../class/TxDecoder");
const util_1 = require("@ethereumjs/util");
const CollectorSocketconnection_1 = require("../log_subscription/CollectorSocketconnection");
exports.receiptsMap = new Map();
async function insertReceipt(receipt) {
    try {
        const fields = Object.keys(receipt).join(', ');
        const placeholders = Object.keys(receipt).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValues)(receipt);
        const sql = 'INSERT OR REPLACE INTO receipts (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(sql, values);
        if (config_1.config.verbose)
            console.log('Successfully inserted Receipt', receipt.receiptId);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to insert Receipt or it is already stored in to database', receipt.receiptId);
    }
}
exports.insertReceipt = insertReceipt;
async function bulkInsertReceipts(receipts) {
    try {
        const fields = Object.keys(receipts[0]).join(', ');
        const placeholders = Object.keys(receipts[0]).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValuesFromArray)(receipts);
        let sql = 'INSERT OR REPLACE INTO receipts (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < receipts.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(sql, values);
        console.log('Successfully bulk inserted receipts', receipts.length);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to bulk insert receipts', receipts.length);
    }
}
exports.bulkInsertReceipts = bulkInsertReceipts;
async function processReceiptData(receipts, saveOnlyNewData = false) {
    var _a, _b, _c, _d;
    if (receipts && receipts.length <= 0)
        return;
    const bucketSize = 1000;
    let combineReceipts = [];
    let combineAccounts1 = [];
    let combineTransactions = [];
    let combineTokenTransactions = []; // For TransactionType (Internal ,ERC20, ERC721)
    let combineTokenTransactions2 = []; // For TransactionType (ERC1155)
    let combineTokens = []; // For Tokens owned by an address
    let accountHistoryStateList = [];
    for (const receiptObj of receipts) {
        const { afterStates, cycle, appReceiptData, tx, timestamp, signedReceipt } = receiptObj;
        if (exports.receiptsMap.has(tx.txId) && exports.receiptsMap.get(tx.txId) === timestamp) {
            continue;
        }
        let modifiedReceiptObj = {
            ...receiptObj,
            beforeStates: config_1.config.storeReceiptBeforeStates ? receiptObj.beforeStates : [],
        };
        if (saveOnlyNewData) {
            const receiptExist = await queryReceiptByReceiptId(tx.txId);
            if (!receiptExist)
                combineReceipts.push(modifiedReceiptObj);
        }
        else
            combineReceipts.push(modifiedReceiptObj);
        let txReceipt = appReceiptData;
        exports.receiptsMap.set(tx.txId, tx.timestamp);
        // Forward receipt data to LogServer
        if (config_1.config.enableCollectorSocketServer)
            await (0, CollectorSocketconnection_1.forwardReceiptData)([receiptObj]);
        // Receipts size can be big, better to save per 100
        if (combineReceipts.length >= 100) {
            await bulkInsertReceipts(combineReceipts);
            combineReceipts = [];
        }
        if (!config_1.config.processData.indexReceipt)
            continue;
        const storageKeyValueMap = {};
        for (const account of afterStates) {
            const accountType = account.data.accountType;
            const accObj = {
                accountId: account.accountId,
                cycle: cycle,
                timestamp: account.timestamp,
                account: account.data,
                hash: account.hash,
                accountType,
                isGlobal: account.isGlobal,
            };
            if (accountType === types_1.AccountType.Account ||
                accountType === types_1.AccountType.ContractStorage ||
                accountType === types_1.AccountType.ContractCode) {
                accObj.ethAddress = account.data.ethAddress.toLowerCase();
                if (config_1.config.processData.decodeContractInfo &&
                    accountType === types_1.AccountType.Account &&
                    'account' in accObj.account &&
                    (0, util_1.bytesToHex)(Uint8Array.from(Object.values(accObj.account.account.codeHash))) !==
                        AccountDB.EOA_CodeHash) {
                    const accountExist = await AccountDB.queryAccountByAccountId(accObj.accountId);
                    if (config_1.config.verbose)
                        console.log('accountExist', accountExist);
                    if (!accountExist) {
                        const { contractInfo, contractType } = await (0, TxDecoder_1.getContractInfo)(accObj.ethAddress);
                        accObj.contractInfo = contractInfo;
                        accObj.contractType = contractType;
                        await AccountDB.insertAccount(accObj);
                    }
                    else {
                        if (accountExist.timestamp < accObj.timestamp) {
                            await AccountDB.updateAccount(accObj.accountId, accObj);
                        }
                    }
                    continue;
                }
                if (config_1.config.processData.decodeTokenTransfer &&
                    accountType === types_1.AccountType.ContractStorage &&
                    'key' in accObj.account) {
                    storageKeyValueMap[accObj.account.key + accObj.ethAddress] = accObj.account;
                }
            }
            else if (accountType === types_1.AccountType.NetworkAccount ||
                accountType === types_1.AccountType.DevAccount ||
                accountType === types_1.AccountType.NodeAccount ||
                accountType === types_1.AccountType.NodeAccount2 ||
                accountType === types_1.AccountType.SecureAccount) {
                accObj.ethAddress = account.accountId; // Adding accountId as ethAddess for these account types for now; since we need ethAddress for mysql index
            }
            const index = combineAccounts1.findIndex((a) => {
                return a.accountId === accObj.accountId;
            });
            if (index > -1) {
                // eslint-disable-next-line security/detect-object-injection
                const accountExist = combineAccounts1[index];
                if (accountExist.timestamp < accObj.timestamp) {
                    combineAccounts1.splice(index, 1);
                    combineAccounts1.push(accObj);
                }
            }
            else {
                const accountExist = await AccountDB.queryAccountByAccountId(accObj.accountId);
                if (config_1.config.verbose)
                    console.log('accountExist', accountExist);
                if (!accountExist) {
                    combineAccounts1.push(accObj);
                }
                else {
                    if (accountExist.timestamp < accObj.timestamp) {
                        await AccountDB.updateAccount(accObj.accountId, accObj);
                    }
                }
            }
            if (accountType === types_1.AccountType.Receipt ||
                accountType === types_1.AccountType.NodeRewardReceipt ||
                accountType === types_1.AccountType.StakeReceipt ||
                accountType === types_1.AccountType.UnstakeReceipt ||
                accountType === types_1.AccountType.InternalTxReceipt)
                txReceipt = account;
        }
        let blockNumber;
        let blockHash;
        if (txReceipt) {
            // if (txReceipt.data.accountType !== AccountType.InternalTxReceipt) {
            const transactionType = txReceipt.data.accountType === types_1.AccountType.Receipt
                ? types_1.TransactionType.Receipt
                : txReceipt.data.accountType === types_1.AccountType.NodeRewardReceipt
                    ? types_1.TransactionType.NodeRewardReceipt
                    : txReceipt.data.accountType === types_1.AccountType.StakeReceipt
                        ? types_1.TransactionType.StakeReceipt
                        : txReceipt.data.accountType === types_1.AccountType.UnstakeReceipt
                            ? types_1.TransactionType.UnstakeReceipt
                            : txReceipt.data.accountType === types_1.AccountType.InternalTxReceipt
                                ? types_1.TransactionType.InternalTxReceipt
                                : -1;
            blockHash = (_b = (_a = txReceipt.data) === null || _a === void 0 ? void 0 : _a.readableReceipt) === null || _b === void 0 ? void 0 : _b.blockHash;
            if (!blockHash)
                console.error(`Transaction ${tx.txId} has no blockHash`);
            blockNumber = parseInt((_d = (_c = txReceipt.data) === null || _c === void 0 ? void 0 : _c.readableReceipt) === null || _d === void 0 ? void 0 : _d.blockNumber);
            if (transactionType !== -1) {
                const txObj = {
                    txId: tx.txId,
                    cycle: cycle,
                    blockNumber,
                    blockHash,
                    timestamp: tx.timestamp,
                    wrappedEVMAccount: txReceipt.data,
                    transactionType,
                    txHash: txReceipt.data.ethAddress,
                    txFrom: txReceipt.data.readableReceipt.from,
                    txTo: txReceipt.data.readableReceipt.to
                        ? txReceipt.data.readableReceipt.to
                        : txReceipt.data.readableReceipt.contractAddress,
                    originalTxData: tx.originalTxData || {},
                };
                if (txReceipt.data.readableReceipt.stakeInfo) {
                    txObj.nominee = txReceipt.data.readableReceipt.stakeInfo.nominee;
                }
                let newTx = true;
                const transactionExist = await TransactionDB.queryTransactionByTxId(tx.txId);
                if (config_1.config.verbose)
                    console.log('transactionExist', transactionExist);
                if (!transactionExist) {
                    if (txObj.nominee)
                        await TransactionDB.insertTransaction(txObj);
                    else
                        combineTransactions.push(txObj);
                }
                else {
                    if (transactionExist.timestamp < txObj.timestamp) {
                        await TransactionDB.insertTransaction(txObj);
                    }
                    newTx = false;
                }
                const { txs, accs, tokens } = await (0, TxDecoder_1.decodeTx)(txObj, storageKeyValueMap, newTx);
                for (const acc of accs) {
                    if (acc === TxDecoder_1.ZERO_ETH_ADDRESS)
                        continue;
                    if (!combineAccounts1.some((a) => a.ethAddress === acc)) {
                        const addressToCreate = acc;
                        const accountExist = await AccountDB.queryAccountByAccountId(addressToCreate.slice(2).toLowerCase() + '0'.repeat(24) //Search by Shardus address
                        );
                        if (config_1.config.verbose)
                            console.log('addressToCreate', addressToCreate, accountExist);
                        if (!accountExist) {
                            // Although this account is not created yet in Shardeum, we created it as a dummy account, so that we can show the account info on the explorer
                            const accObj = {
                                accountId: addressToCreate.slice(2).toLowerCase() + '0'.repeat(24),
                                cycle: txObj.cycle,
                                timestamp: txObj.timestamp,
                                ethAddress: addressToCreate,
                                account: {
                                    nonce: '0',
                                    balance: '0',
                                },
                                hash: 'Ox',
                                accountType: types_1.AccountType.Account,
                                isGlobal: false,
                            };
                            combineAccounts1.push(accObj);
                        }
                    }
                }
                for (const tx of txs) {
                    let accountExist = null;
                    if (tx.tokenType !== types_1.TransactionType.EVM_Internal)
                        accountExist = await AccountDB.queryAccountByAccountId(tx.contractAddress.slice(2).toLowerCase() + '0'.repeat(24) //Search by Shardus address
                        );
                    let contractInfo = {};
                    if (accountExist && accountExist.contractInfo) {
                        contractInfo = accountExist.contractInfo;
                    }
                    if ('amountSpent' in txObj.wrappedEVMAccount) {
                        const obj = {
                            ...tx,
                            txId: txObj.txId,
                            txHash: txObj.txHash,
                            cycle: txObj.cycle,
                            timestamp: txObj.timestamp,
                            transactionFee: txObj.wrappedEVMAccount.amountSpent,
                            contractInfo,
                        };
                        if (tx.tokenType === types_1.TransactionType.ERC_1155) {
                            combineTokenTransactions2.push(obj);
                        }
                        else {
                            combineTokenTransactions.push(obj);
                        }
                    }
                }
                combineTokens = [...combineTokens, ...tokens];
            }
        }
        if (config_1.config.saveAccountHistoryState) {
            if (receiptObj.globalModification === false &&
                signedReceipt &&
                blockNumber &&
                blockHash &&
                signedReceipt.proposal.accountIDs.length > 0) {
                for (let i = 0; i < signedReceipt.proposal.accountIDs.length; i++) {
                    const accountHistoryState = {
                        accountId: signedReceipt.proposal.accountIDs[i],
                        beforeStateHash: signedReceipt.proposal.beforeStateHashes[i],
                        afterStateHash: signedReceipt.proposal.afterStateHashes[i],
                        timestamp,
                        blockNumber,
                        blockHash,
                        receiptId: tx.txId,
                    };
                    accountHistoryStateList.push(accountHistoryState);
                }
            }
            else {
                if (receiptObj.globalModification === true) {
                    console.log(`Receipt ${tx.txId} with timestamp ${timestamp} has globalModification as true`);
                }
                if (receiptObj.globalModification === false && !signedReceipt) {
                    console.error(`Receipt ${tx.txId} with timestamp ${timestamp} has no signedReceipt`);
                }
                if (!blockNumber || !blockHash) {
                    console.error(`Receipt ${tx.txId} with timestamp ${timestamp} has no blockNumber or blockHash`);
                }
            }
        }
        if (combineAccounts1.length >= bucketSize) {
            await AccountDB.bulkInsertAccounts(combineAccounts1);
            combineAccounts1 = [];
        }
        if (combineTransactions.length >= bucketSize) {
            await TransactionDB.bulkInsertTransactions(combineTransactions);
            combineTransactions = [];
        }
        if (combineTokenTransactions.length >= bucketSize) {
            await TransactionDB.bulkInsertTokenTransactions(combineTokenTransactions);
            combineTokenTransactions = [];
        }
        if (combineTokenTransactions2.length >= bucketSize) {
            await TransactionDB.bulkInsertTokenTransactions(combineTokenTransactions2);
            combineTokenTransactions2 = [];
        }
        if (combineTokens.length >= bucketSize) {
            await AccountDB.bulkInsertTokens(combineTokens);
            combineTokens = [];
        }
        if (accountHistoryStateList.length > bucketSize) {
            await AccountHistoryStateDB.bulkInsertAccountHistoryStates(accountHistoryStateList);
            accountHistoryStateList = [];
        }
    }
    if (combineReceipts.length > 0)
        await bulkInsertReceipts(combineReceipts);
    if (combineAccounts1.length > 0)
        await AccountDB.bulkInsertAccounts(combineAccounts1);
    if (combineTransactions.length > 0)
        await TransactionDB.bulkInsertTransactions(combineTransactions);
    if (combineTokenTransactions.length > 0)
        await TransactionDB.bulkInsertTokenTransactions(combineTokenTransactions);
    if (combineTokenTransactions2.length > 0)
        await TransactionDB.bulkInsertTokenTransactions(combineTokenTransactions2);
    if (combineTokens.length > 0)
        await AccountDB.bulkInsertTokens(combineTokens);
    if (accountHistoryStateList.length > 0)
        await AccountHistoryStateDB.bulkInsertAccountHistoryStates(accountHistoryStateList);
}
exports.processReceiptData = processReceiptData;
async function queryReceiptByReceiptId(receiptId) {
    try {
        const sql = `SELECT * FROM receipts WHERE receiptId=?`;
        const receipt = await db.get(sql, [receiptId]);
        if (receipt)
            deserializeDbReceipt(receipt);
        if (config_1.config.verbose)
            console.log('Receipt receiptId', receipt);
        return receipt;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryReceiptByReceiptId = queryReceiptByReceiptId;
async function queryLatestReceipts(count) {
    try {
        const sql = `SELECT * FROM receipts ORDER BY cycle DESC, timestamp DESC LIMIT ${count}`;
        const receipts = await db.all(sql);
        receipts.forEach((receipt) => deserializeDbReceipt(receipt));
        if (config_1.config.verbose)
            console.log('Receipt latest', receipts);
        return receipts;
    }
    catch (e) {
        console.log(e);
    }
    return [];
}
exports.queryLatestReceipts = queryLatestReceipts;
async function queryReceipts(skip = 0, limit = 10000) {
    let receipts = [];
    try {
        const sql = `SELECT * FROM receipts ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        receipts = await db.all(sql);
        receipts.forEach((receipt) => deserializeDbReceipt(receipt));
    }
    catch (e) {
        console.log(e);
    }
    if (config_1.config.verbose)
        console.log('Receipt receipts', receipts ? receipts.length : receipts, 'skip', skip);
    return receipts;
}
exports.queryReceipts = queryReceipts;
async function queryReceiptCount() {
    let receipts = { 'COUNT(*)': 0 };
    try {
        const sql = `SELECT COUNT(*) FROM receipts`;
        receipts = await db.get(sql, []);
    }
    catch (e) {
        console.log(e);
    }
    if (config_1.config.verbose)
        console.log('Receipt count', receipts);
    return receipts['COUNT(*)'] || 0;
}
exports.queryReceiptCount = queryReceiptCount;
async function queryReceiptCountByCycles(start, end) {
    let receipts = [];
    try {
        const sql = `SELECT cycle, COUNT(*) FROM receipts GROUP BY cycle HAVING cycle BETWEEN ? AND ? ORDER BY cycle ASC`;
        receipts = await db.all(sql, [start, end]);
    }
    catch (e) {
        console.log(e);
    }
    if (config_1.config.verbose)
        console.log('Receipt count by cycles', receipts);
    return receipts.map((receipt) => {
        return {
            receipts: receipt['COUNT(*)'],
            cycle: receipt.cycle,
        };
    });
}
exports.queryReceiptCountByCycles = queryReceiptCountByCycles;
async function queryReceiptsBetweenCycles(skip = 0, limit = 10, start, end) {
    let receipts = [];
    try {
        const sql = `SELECT * FROM receipts WHERE cycle BETWEEN ? and ? ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        receipts = await db.all(sql, [start, end]);
        receipts.forEach((receipt) => deserializeDbReceipt(receipt));
    }
    catch (e) {
        console.log(e);
    }
    if (config_1.config.verbose)
        console.log('Receipt receipts between cycles', receipts);
    return receipts;
}
exports.queryReceiptsBetweenCycles = queryReceiptsBetweenCycles;
async function queryReceiptCountBetweenCycles(start, end) {
    let receipts = { 'COUNT(*)': 0 };
    try {
        const sql = `SELECT COUNT(*) FROM receipts WHERE cycle BETWEEN ? and ?`;
        receipts = await db.get(sql, [start, end]);
    }
    catch (e) {
        console.log(e);
    }
    if (config_1.config.verbose)
        console.log('Receipt receipts count between cycles', receipts);
    return receipts['COUNT(*)'] || 0;
}
exports.queryReceiptCountBetweenCycles = queryReceiptCountBetweenCycles;
function deserializeDbReceipt(receipt) {
    receipt.tx && (receipt.tx = lib_types_1.Utils.safeJsonParse(receipt.tx));
    receipt.beforeStates && (receipt.beforeStates = lib_types_1.Utils.safeJsonParse(receipt.beforeStates));
    receipt.afterStates && (receipt.afterStates = lib_types_1.Utils.safeJsonParse(receipt.afterStates));
    receipt.appReceiptData && (receipt.appReceiptData = lib_types_1.Utils.safeJsonParse(receipt.appReceiptData));
    receipt.signedReceipt && (receipt.signedReceipt = lib_types_1.Utils.safeJsonParse(receipt.signedReceipt));
    // globalModification is stored as 0 or 1 in the database, convert it to boolean
    receipt.globalModification = receipt.globalModification === 1;
}
function cleanOldReceiptsMap(timestamp) {
    for (const [key, value] of exports.receiptsMap) {
        if (value < timestamp)
            exports.receiptsMap.delete(key);
    }
    if (config_1.config.verbose)
        console.log('Clean Old Receipts Map', timestamp, exports.receiptsMap);
}
exports.cleanOldReceiptsMap = cleanOldReceiptsMap;
//# sourceMappingURL=receipt.js.map