"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractValuesFromArray = exports.extractValues = exports.close = exports.all = exports.get = exports.run = exports.runCreate = exports.init = void 0;
const sqlite3_1 = __importDefault(require("sqlite3"));
const sqlite3 = sqlite3_1.default.verbose();
const config_1 = require("../config");
const lib_types_1 = require("@shardeum-foundation/lib-types");
let db;
// Additional databases
let shardeumIndexerDb;
async function init(config) {
    db = new sqlite3.Database(config.defaultDbSqlitePath);
    await run('PRAGMA journal_mode=WAL');
    console.log('Database initialized.');
    if (config.enableShardeumIndexer) {
        shardeumIndexerDb = new sqlite3.Database(config.shardeumIndexerSqlitePath);
        await run('PRAGMA journal_mode=WAL', [], 'shardeumIndexer');
        console.log('Shardeum indexer database initialized.');
    }
    db.on('profile', (sql, time) => {
        if (time > 500 && time < 1000) {
            console.log('SLOW QUERY', sql, time);
        }
        else if (time > 1000) {
            console.log('VERY SLOW QUERY', sql, time);
        }
    });
}
exports.init = init;
function getDb(dbName) {
    switch (dbName) {
        case 'default':
            return db;
        case 'shardeumIndexer':
            return shardeumIndexerDb;
    }
}
async function runCreate(createStatement, dbName = 'default') {
    await run(createStatement, [], dbName);
}
exports.runCreate = runCreate;
async function run(sql, params = [], dbName = 'default') {
    return new Promise((resolve, reject) => {
        getDb(dbName).run(sql, params, function (err) {
            if (err) {
                console.log('Error running sql ' + sql);
                console.log(err);
                reject(err);
            }
            else {
                resolve({ id: this.lastID });
            }
        });
    });
}
exports.run = run;
async function get(sql, params = [], dbName = 'default') {
    return new Promise((resolve, reject) => {
        getDb(dbName).get(sql, params, (err, result) => {
            if (err) {
                console.log('Error running sql: ' + sql);
                console.log(err);
                reject(err);
            }
            else {
                resolve(result);
            }
        });
    });
}
exports.get = get;
async function all(sql, params = [], dbName = 'default') {
    return new Promise((resolve, reject) => {
        getDb(dbName).all(sql, params, (err, rows) => {
            if (err) {
                console.log('Error running sql: ' + sql);
                console.log(err);
                reject(err);
            }
            else {
                resolve(rows);
            }
        });
    });
}
exports.all = all;
/**
 * Closes the Database and Indexer Connections Gracefully
 */
async function close() {
    try {
        console.log('Terminating Database/Indexer Connections...');
        await new Promise((resolve, reject) => {
            db.close((err) => {
                if (err) {
                    console.error('Error closing Database Connection.');
                    reject(err);
                }
                else {
                    console.log('Database connection closed.');
                    resolve();
                }
            });
        });
        if (config_1.config.enableShardeumIndexer && shardeumIndexerDb) {
            await new Promise((resolve, reject) => {
                shardeumIndexerDb.close((err) => {
                    if (err) {
                        console.error('Error closing Indexer Connection.');
                        reject(err);
                    }
                    else {
                        console.log('Shardeum Indexer Database Connection closed.');
                        resolve();
                    }
                });
            });
        }
    }
    catch (err) {
        console.error('Error thrown in db close() function: ');
        console.error(err);
    }
}
exports.close = close;
function extractValues(object) {
    try {
        const inputs = [];
        for (let value of Object.values(object)) {
            if (typeof value === 'object')
                value = lib_types_1.Utils.safeStringify(value);
            inputs.push(value);
        }
        return inputs;
    }
    catch (e) {
        console.log(e);
    }
    return [];
}
exports.extractValues = extractValues;
function extractValuesFromArray(arr) {
    try {
        const inputs = [];
        for (const object of arr) {
            for (let value of Object.values(object)) {
                if (typeof value === 'object')
                    value = lib_types_1.Utils.safeStringify(value);
                inputs.push(value);
            }
        }
        return inputs;
    }
    catch (e) {
        console.log(e);
    }
    return [];
}
exports.extractValuesFromArray = extractValuesFromArray;
//# sourceMappingURL=sqlite3storage.js.map