import { Transaction, TokenTx, TransactionSearchType, WrappedEVMAccount, InternalTXType } from '../types';
import Web3 from 'web3';
export { type Transaction } from '../types';
export declare const ERC20_METHOD_DIC: {
    '0xa9059cbb': string;
    '0xa978501e': string;
};
type DbTransaction = Transaction & {
    wrappedEVMAccount: string;
    originalTxData: string;
};
type DbTokenTx = TokenTx & {
    contractInfo: string;
};
export declare function insertTransaction(transaction: Transaction): Promise<void>;
export declare function bulkInsertTransactions(transactions: Transaction[]): Promise<void>;
export declare function updateTransaction(_txId: string, transaction: Partial<Transaction>): Promise<void>;
export declare function insertTokenTransaction(tokenTx: TokenTx): Promise<void>;
export declare function bulkInsertTokenTransactions(tokenTxs: TokenTx[]): Promise<void>;
interface RawTransaction {
    accountId: string;
    cycleNumber: number;
    data: WrappedEVMAccount;
    originalTxData: {
        duration: number;
        internalTXType: InternalTXType;
        isInternalTx: boolean;
        nominator: string;
        nominee: string;
        sign: {
            owner: string;
            sig: string;
        };
        timestamp: number;
    };
    result: {
        txIdShort: string;
        txResult: string;
    };
    sign: {
        owner: string;
        sig: string;
    };
    timestamp: number;
    txId: string;
}
export declare function processTransactionData(transactions: RawTransaction[]): Promise<void>;
export declare const getWeb3: () => Promise<Web3>;
export declare function queryTransactionCount(address?: string, txType?: TransactionSearchType, filterAddress?: string): Promise<number>;
export declare function queryTransactions(skip?: number, limit?: number, address?: string, txType?: TransactionSearchType, filterAddress?: string): Promise<(DbTransaction | DbTokenTx)[]>;
export declare function queryTransactionByTxId(txId: string, detail?: boolean): Promise<Transaction | null>;
export declare function queryTransactionByHash(txHash: string, detail?: boolean): Promise<Transaction[] | null>;
export declare function queryTransactionsForCycle(cycleNumber: number): Promise<Transaction[]>;
export declare function queryTransactionsBetweenCycles(skip: number, limit: number, start: number, end: number, address?: string, txType?: TransactionSearchType, filterAddress?: string): Promise<(DbTransaction | DbTokenTx)[]>;
export declare function queryTransactionCountBetweenCycles(start: number, end: number, address?: string, txType?: TransactionSearchType, filterAddress?: string): Promise<number>;
export declare function queryTransactionCountByCycles(start: number, end: number, txType?: TransactionSearchType): Promise<{
    cycle: number;
    transactions: number;
}[]>;
export declare function queryTransactionCountByTimestamp(beforeTimestamp: number, afterTimestamp: number, address?: string, txType?: TransactionSearchType, filterAddress?: string): Promise<number>;
export declare function queryTransactionsByTimestamp(skip: number, limit: number, beforeTimestamp: number, afterTimestamp: number, address?: string, txType?: TransactionSearchType, filterAddress?: string): Promise<(DbTransaction | DbTokenTx)[]>;
export declare function queryTransactionCountByBlock(blockNumber: number, blockHash: string): Promise<number>;
export declare function queryTransactionsByBlock(blockNumber: number, blockHash: string): Promise<DbTransaction[]>;
export declare function queryTokenTxByTxId(txId: string): Promise<DbTokenTx[] | []>;
