"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryTokenTxByTxId = exports.queryTransactionsByBlock = exports.queryTransactionCountByBlock = exports.queryTransactionsByTimestamp = exports.queryTransactionCountByTimestamp = exports.queryTransactionCountByCycles = exports.queryTransactionCountBetweenCycles = exports.queryTransactionsBetweenCycles = exports.queryTransactionsForCycle = exports.queryTransactionByHash = exports.queryTransactionByTxId = exports.queryTransactions = exports.queryTransactionCount = exports.getWeb3 = exports.processTransactionData = exports.bulkInsertTokenTransactions = exports.insertTokenTransaction = exports.updateTransaction = exports.bulkInsertTransactions = exports.insertTransaction = exports.ERC20_METHOD_DIC = void 0;
const db = __importStar(require("./sqlite3storage"));
const sqlite3storage_1 = require("./sqlite3storage");
const index_1 = require("../config/index");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const types_1 = require("../types");
const web3_1 = __importDefault(require("web3"));
const AccountDB = __importStar(require("./account"));
const TxDecoder_1 = require("../class/TxDecoder");
exports.ERC20_METHOD_DIC = {
    '0xa9059cbb': 'transfer',
    '0xa978501e': 'transferFrom',
};
async function insertTransaction(transaction) {
    try {
        const fields = Object.keys(transaction).join(', ');
        const placeholders = Object.keys(transaction).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValues)(transaction);
        const sql = 'INSERT OR REPLACE INTO transactions (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(sql, values);
        if (index_1.config.verbose)
            console.log('Successfully inserted Transaction', transaction.txId, transaction.txHash);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to insert Transaction or it is already stored in to database', transaction.txId);
    }
}
exports.insertTransaction = insertTransaction;
async function bulkInsertTransactions(transactions) {
    try {
        const fields = Object.keys(transactions[0]).join(', ');
        const placeholders = Object.keys(transactions[0]).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValuesFromArray)(transactions);
        let sql = 'INSERT OR REPLACE INTO transactions (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < transactions.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(sql, values);
        console.log('Successfully bulk inserted transactions', transactions.length);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to bulk insert transactions', transactions.length);
    }
}
exports.bulkInsertTransactions = bulkInsertTransactions;
async function updateTransaction(_txId, transaction) {
    try {
        const sql = `UPDATE transactions SET result = $result, cycle = $cycle, wrappedEVMAccount = $wrappedEVMAccount, txHash = $txHash WHERE txId = $txId `;
        await db.run(sql, {
            $cycle: transaction.cycle,
            $wrappedEVMAccount: transaction.wrappedEVMAccount && lib_types_1.Utils.safeStringify(transaction.wrappedEVMAccount),
            $txHash: transaction.txHash,
            $txId: transaction.txId,
        });
        if (index_1.config.verbose)
            console.log('Successfully Updated Transaction', transaction.txId, transaction.txHash);
    }
    catch (e) {
        /* prettier-ignore */ if (index_1.config.verbose)
            console.log(e);
        console.log('Unable to update Transaction', transaction.txId, transaction.txHash);
    }
}
exports.updateTransaction = updateTransaction;
async function insertTokenTransaction(tokenTx) {
    try {
        const fields = Object.keys(tokenTx).join(', ');
        const placeholders = Object.keys(tokenTx).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValues)(tokenTx);
        const sql = 'INSERT OR REPLACE INTO tokenTxs (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(sql, values);
        if (index_1.config.verbose)
            console.log('Successfully inserted Token Transaction', tokenTx.txHash);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to insert Token Transaction or it is already stored in to database', tokenTx.txHash);
    }
}
exports.insertTokenTransaction = insertTokenTransaction;
async function bulkInsertTokenTransactions(tokenTxs) {
    try {
        const fields = Object.keys(tokenTxs[0]).join(', ');
        const placeholders = Object.keys(tokenTxs[0]).fill('?').join(', ');
        const values = (0, sqlite3storage_1.extractValuesFromArray)(tokenTxs);
        let sql = 'INSERT OR REPLACE INTO tokenTxs (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < tokenTxs.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(sql, values);
        console.log('Successfully inserted token transactions', tokenTxs.length);
    }
    catch (e) {
        console.log(e);
        console.log('Unable to bulk insert token transactions', tokenTxs.length);
    }
}
exports.bulkInsertTokenTransactions = bulkInsertTokenTransactions;
function isReceiptData(obj) {
    const accountType = obj === null || obj === void 0 ? void 0 : obj.accountType;
    return (accountType === types_1.AccountType.Receipt ||
        accountType === types_1.AccountType.NodeRewardReceipt ||
        accountType === types_1.AccountType.StakeReceipt ||
        accountType === types_1.AccountType.UnstakeReceipt ||
        accountType === types_1.AccountType.InternalTxReceipt);
}
async function processTransactionData(transactions) {
    var _a, _b;
    console.log('transactions size', transactions.length);
    if (transactions && transactions.length <= 0)
        return;
    const bucketSize = 1000;
    const combineAccounts = [];
    const existingAccounts = []; // To save perf on querying from the db again and again, save the existing account that is queried once in memory
    let combineTransactions = [];
    let combineTokenTransactions = []; // For TransactionType (Internal ,ERC20, ERC721)
    let combineTokenTransactions2 = []; // For TransactionType (ERC1155)
    let combineTokens = []; // For Tokens owned by an address
    for (const transaction of transactions) {
        if (isReceiptData(transaction.data)) {
            const txObj = {
                txId: (_a = transaction.data) === null || _a === void 0 ? void 0 : _a.txId,
                cycle: transaction.cycleNumber,
                blockNumber: parseInt(transaction.data.readableReceipt.blockNumber),
                blockHash: transaction.data.readableReceipt.blockHash,
                timestamp: transaction.timestamp,
                wrappedEVMAccount: transaction.data,
                transactionType: transaction.data.accountType === types_1.AccountType.Receipt
                    ? types_1.TransactionType.Receipt
                    : transaction.data.accountType === types_1.AccountType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : transaction.data.accountType === types_1.AccountType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : transaction.data.accountType === types_1.AccountType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt,
                txHash: transaction.data.ethAddress,
                txFrom: transaction.data.readableReceipt.from,
                txTo: transaction.data.readableReceipt.to
                    ? transaction.data.readableReceipt.to
                    : transaction.data.readableReceipt.contractAddress,
                originalTxData: {},
            };
            const { txs, accs, tokens } = await (0, TxDecoder_1.decodeTx)(txObj);
            for (const acc of accs) {
                if (acc === TxDecoder_1.ZERO_ETH_ADDRESS)
                    continue;
                const index = combineAccounts.findIndex((a) => a.accountId === acc.slice(2).toLowerCase() + '0'.repeat(24));
                if (index > -1) {
                    // eslint-disable-next-line security/detect-object-injection
                    const accountExist = combineAccounts[index];
                    accountExist.timestamp = txObj.timestamp;
                    combineAccounts.splice(index, 1);
                    combineAccounts.push(accountExist);
                }
                else {
                    const addressToCreate = acc;
                    // To save performance on querying from the db again and again, save it in memory
                    if (existingAccounts.includes(addressToCreate)) {
                        continue;
                    }
                    const accountExist = await AccountDB.queryAccountByAccountId(addressToCreate.slice(2).toLowerCase() + '0'.repeat(24) //Search by Shardus address
                    );
                    if (index_1.config.verbose)
                        console.log('addressToCreate', addressToCreate, accountExist);
                    if (!accountExist) {
                        // Although this account is not created yet in Shardeum, we created it as a dummy account, so that we can show the account info on the explorer
                        const accObj = {
                            accountId: addressToCreate.slice(2).toLowerCase() + '0'.repeat(24),
                            cycle: txObj.cycle,
                            timestamp: txObj.timestamp,
                            ethAddress: addressToCreate,
                            account: {
                                nonce: '0',
                                balance: '0',
                            },
                            hash: 'Ox',
                            accountType: types_1.AccountType.Account,
                            isGlobal: false,
                        };
                        combineAccounts.push(accObj);
                    }
                    else {
                        existingAccounts.push(addressToCreate);
                    }
                }
            }
            for (const tx of txs) {
                const accountExist = await AccountDB.queryAccountByAccountId(tx.contractAddress.slice(2).toLowerCase() + '0'.repeat(24) //Search by Shardus address
                );
                let contractInfo = {};
                if (accountExist && accountExist.contractInfo) {
                    contractInfo = accountExist.contractInfo;
                }
                // wrapped data must be a receipt here. this type guard ensures that
                if ('readableReceipt' in txObj.wrappedEVMAccount) {
                    const obj = {
                        ...tx,
                        txId: txObj.txId,
                        txHash: txObj.txHash,
                        cycle: txObj.cycle,
                        timestamp: txObj.timestamp,
                        transactionFee: (_b = txObj.wrappedEVMAccount.readableReceipt.gasUsed) !== null && _b !== void 0 ? _b : '0',
                        contractInfo,
                    };
                    if (tx.tokenType === types_1.TransactionType.ERC_1155) {
                        combineTokenTransactions2.push(obj);
                    }
                    else {
                        combineTokenTransactions.push(obj);
                    }
                }
            }
            combineTokens = [...combineTokens, ...tokens];
            combineTransactions.push(txObj);
        }
        if (combineTransactions.length >= bucketSize) {
            await bulkInsertTransactions(combineTransactions);
            combineTransactions = [];
        }
        if (combineTokenTransactions.length >= bucketSize) {
            await bulkInsertTokenTransactions(combineTokenTransactions);
            combineTokenTransactions = [];
        }
        if (combineTokenTransactions2.length >= bucketSize) {
            await bulkInsertTokenTransactions(combineTokenTransactions2);
            combineTokenTransactions2 = [];
        }
        if (combineTokens.length >= bucketSize) {
            await AccountDB.bulkInsertTokens(combineTokens);
            combineTokens = [];
        }
    }
    if (combineAccounts.length > 0) {
        let limit = bucketSize;
        let j = limit;
        let accountsToSave;
        for (let i = 0; i < combineAccounts.length; i = j) {
            accountsToSave = combineAccounts.slice(i, limit);
            await AccountDB.bulkInsertAccounts(accountsToSave);
            j = limit;
            limit += bucketSize;
        }
    }
    if (combineTransactions.length > 0)
        await bulkInsertTransactions(combineTransactions);
    if (combineTokenTransactions.length > 0)
        await bulkInsertTokenTransactions(combineTokenTransactions);
    if (combineTokenTransactions2.length > 0)
        await bulkInsertTokenTransactions(combineTokenTransactions2);
    if (combineTokens.length > 0)
        await AccountDB.bulkInsertTokens(combineTokens);
}
exports.processTransactionData = processTransactionData;
const getWeb3 = function () {
    return new Promise((resolve, reject) => {
        try {
            const web3 = new web3_1.default(new web3_1.default.providers.HttpProvider(`${index_1.config.rpcUrl}`));
            resolve(web3);
        }
        catch (e) {
            console.error(e);
            reject('Cannot get web3 instance');
        }
    });
};
exports.getWeb3 = getWeb3;
async function queryTransactionCount(address, txType, filterAddress) {
    let transactions = { 'COUNT(*)': 0 };
    try {
        if (address) {
            if (!txType) {
                const sql = `SELECT COUNT(*) FROM transactions WHERE txFrom=? OR txTo=? OR nominee=?`;
                transactions = await db.get(sql, [address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                const ty = types_1.TransactionType.InternalTxReceipt;
                const sql = `SELECT COUNT(*) FROM transactions WHERE transactionType!=? AND (txFrom=? OR txTo=? OR nominee=?)`;
                transactions = await db.get(sql, [ty, address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                let sql = `SELECT COUNT(*) FROM transactions WHERE transactionType=? AND (txFrom=? OR txTo=? OR nominee=?)`;
                if (txType === types_1.TransactionSearchType.InternalTxReceipt) {
                    sql = `SELECT COUNT(*) FROM transactions WHERE (transactionType!=? AND transactionType!=? AND transactionType!=?) AND (txFrom=? OR txTo=? OR nominee=?)`;
                    transactions = await db.get(sql, [
                        types_1.TransactionType.Receipt,
                        types_1.TransactionType.StakeReceipt,
                        types_1.TransactionType.UnstakeReceipt,
                        address,
                        address,
                        address,
                    ]);
                }
                else {
                    transactions = await db.get(sql, [ty, address, address, address]);
                }
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                const sql = `SELECT COUNT(*) FROM tokenTxs WHERE (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND tokenType=?`;
                transactions = await db.get(sql, [address, address, address, ty]);
            }
            else if (txType === types_1.TransactionSearchType.TokenTransfer) {
                if (filterAddress) {
                    const sql = `SELECT COUNT(*) FROM tokenTxs WHERE contractAddress=? AND (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND NOT tokenType=?`;
                    transactions = await db.get(sql, [
                        address,
                        filterAddress,
                        filterAddress,
                        filterAddress,
                        types_1.TransactionType.EVM_Internal,
                    ]);
                }
                else {
                    const sql = `SELECT COUNT(*) FROM tokenTxs WHERE contractAddress=? AND NOT tokenType=?`;
                    transactions = await db.get(sql, [address, types_1.TransactionType.EVM_Internal]);
                }
            }
        }
        else if (txType || txType === types_1.TransactionSearchType.All) {
            if (txType === types_1.TransactionSearchType.All) {
                const sql = `SELECT COUNT(*) FROM transactions`;
                transactions = await db.get(sql, []);
            }
            else if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                const sql = `SELECT COUNT(*) FROM transactions WHERE (transactionType=? OR transactionType=? OR transactionType=?)`;
                transactions = await db.get(sql, [
                    types_1.TransactionType.Receipt,
                    types_1.TransactionType.StakeReceipt,
                    types_1.TransactionType.UnstakeReceipt,
                ]);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                const sql = `SELECT COUNT(*) FROM transactions WHERE transactionType=?`;
                transactions = await db.get(sql, [ty]);
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                const sql = `SELECT COUNT(*) FROM tokenTxs WHERE tokenType=?`;
                transactions = await db.get(sql, [ty]);
            }
        }
        else {
            const sql = `SELECT COUNT(*) FROM transactions`;
            transactions = await db.get(sql);
        }
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('transactions count', transactions);
    return transactions['COUNT(*)'] || 0;
}
exports.queryTransactionCount = queryTransactionCount;
async function queryTransactions(skip = 0, limit = 10, address, txType, filterAddress) {
    let transactions = [];
    try {
        if (address) {
            if (!txType) {
                const sql = `SELECT * FROM transactions WHERE (txFrom=? OR txTo=? OR nominee=?) ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                transactions = await db.all(sql, [address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                // const sql = `SELECT * FROM transactions WHERE (transactionType=? OR transactionType=? OR transactionType=?) AND (txFrom=? OR txTo=? OR nominee=?) ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`
                // transactions = await db.all(sql, [
                //   TransactionType.Receipt,
                //   TransactionType.StakeReceipt,
                //   TransactionType.UnstakeReceipt,
                //   address,
                //   address,
                //   address,
                // ])
                const sql = `SELECT * FROM transactions WHERE transactionType!=? AND (txFrom=? OR txTo=? OR nominee=?) ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                transactions = await db.all(sql, [types_1.TransactionType.InternalTxReceipt, address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                let sql = `SELECT * FROM transactions WHERE transactionType=? AND (txFrom=? OR txTo=? OR nominee=?) ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                if (txType === types_1.TransactionSearchType.InternalTxReceipt) {
                    sql = `SELECT * FROM transactions WHERE (transactionType!=? AND transactionType!=? AND transactionType!=?) AND (txFrom=? OR txTo=? OR nominee=?) ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                    transactions = await db.all(sql, [
                        types_1.TransactionType.Receipt,
                        types_1.TransactionType.StakeReceipt,
                        types_1.TransactionType.UnstakeReceipt,
                        address,
                        address,
                        address,
                    ]);
                }
                else
                    transactions = await db.all(sql, [ty, address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                const sql = `SELECT * FROM tokenTxs WHERE (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND tokenType=? ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                transactions = await db.all(sql, [address, address, address, ty]);
            }
            else if (txType === types_1.TransactionSearchType.TokenTransfer) {
                if (filterAddress) {
                    const sql = `SELECT * FROM tokenTxs WHERE contractAddress=? AND (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND NOT (tokenType=?) ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                    transactions = await db.all(sql, [
                        address,
                        filterAddress,
                        filterAddress,
                        filterAddress,
                        types_1.TransactionType.EVM_Internal,
                    ]);
                }
                else {
                    const sql = `SELECT * FROM tokenTxs WHERE contractAddress=? AND NOT (tokenType=?) ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                    transactions = await db.all(sql, [address, types_1.TransactionType.EVM_Internal]);
                }
            }
        }
        else if (txType) {
            if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                const ty = types_1.TransactionType.InternalTxReceipt;
                const sql = `SELECT * FROM transactions WHERE transactionType!=? ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                transactions = await db.all(sql, [ty]);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                let sql = `SELECT * FROM transactions WHERE transactionType=? ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                if (txType === types_1.TransactionSearchType.InternalTxReceipt) {
                    sql = `SELECT * FROM transactions WHERE transactionType!=? AND transactionType!=? AND transactionType!=? ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                    transactions = await db.all(sql, [
                        types_1.TransactionType.Receipt,
                        types_1.TransactionType.StakeReceipt,
                        types_1.TransactionType.UnstakeReceipt,
                    ]);
                }
                else {
                    transactions = await db.all(sql, [ty]);
                }
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                const sql = `SELECT * FROM tokenTxs WHERE tokenType=? ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
                transactions = await db.all(sql, [ty]);
            }
        }
        else {
            const sql = `SELECT * FROM transactions ORDER BY cycle DESC, timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
            transactions = await db.all(sql);
        }
        if (transactions.length > 0) {
            transactions.forEach((transaction) => {
                if ('transactionType' in transaction && transaction.transactionType)
                    deserializeDbTransaction(transaction);
                else if ('tokenType' in transaction && transaction.tokenType)
                    deserializeDbToken(transaction);
            });
        }
        if (index_1.config.verbose)
            console.log('transactions', transactions);
    }
    catch (e) {
        console.log(e);
    }
    return transactions;
}
exports.queryTransactions = queryTransactions;
async function queryTransactionByTxId(txId, detail = false) {
    try {
        const sql = `SELECT * FROM transactions WHERE txId=?`;
        const transaction = await db.get(sql, [txId]);
        if (transaction) {
            deserializeDbTransaction(transaction);
        }
        if (detail) {
            const sql = `SELECT * FROM tokenTxs WHERE txId=?`;
            const tokenTxs = await db.all(sql, [txId]);
            if (tokenTxs.length > 0) {
                tokenTxs.forEach((tokenTx) => deserializeDbToken(tokenTx));
                transaction.tokenTxs = tokenTxs;
            }
        }
        if (index_1.config.verbose)
            console.log('transaction txId', transaction);
        return transaction;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryTransactionByTxId = queryTransactionByTxId;
async function queryTransactionByHash(txHash, detail = false) {
    try {
        const sql = `SELECT * FROM transactions WHERE txHash=? ORDER BY cycle DESC, timestamp DESC`;
        const transactions = await db.all(sql, [txHash]);
        if (transactions.length > 0) {
            for (const transaction of transactions) {
                deserializeDbTransaction(transaction);
                if (detail) {
                    const sql = `SELECT * FROM tokenTxs WHERE txId=? ORDER BY cycle DESC, timestamp DESC`;
                    const tokenTxs = await db.all(sql, [transaction.txId]);
                    if (tokenTxs.length > 0) {
                        tokenTxs.forEach((tokenTx) => deserializeDbToken(tokenTx));
                        transaction.tokenTxs = tokenTxs;
                    }
                }
            }
        }
        if (index_1.config.verbose)
            console.log('transaction hash', transactions);
        return transactions;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryTransactionByHash = queryTransactionByHash;
async function queryTransactionsForCycle(cycleNumber) {
    let transactions = [];
    try {
        const sql = `SELECT * FROM transactions WHERE cycle=? ORDER BY timestamp ASC`;
        transactions = await db.all(sql, [cycleNumber]);
        if (transactions.length > 0) {
            transactions.forEach((transaction) => deserializeDbTransaction(transaction));
        }
        if (index_1.config.verbose)
            console.log('transactions for cycle', cycleNumber, transactions);
    }
    catch (e) {
        console.log('exception when querying transactions for cycle', cycleNumber, e);
    }
    return transactions;
}
exports.queryTransactionsForCycle = queryTransactionsForCycle;
async function queryTransactionsBetweenCycles(skip = 0, limit = 10, start, end, address, txType, filterAddress) {
    let transactions = [];
    try {
        if (address) {
            if (!txType || types_1.TransactionSearchType.All) {
                const sql = `SELECT * FROM transactions WHERE cycle BETWEEN ? and ? AND (txFrom=? OR txTo=? OR nominee=?) ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                transactions = await db.all(sql, [start, end, address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                const ty = types_1.TransactionType.InternalTxReceipt;
                const sql = `SELECT * FROM transactions WHERE cycle BETWEEN ? and ? AND transactionType!=? AND (txFrom=? OR txTo=? OR nominee=?) ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                transactions = await db.all(sql, [start, end, ty, address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                let sql = `SELECT * FROM transactions WHERE cycle BETWEEN ? and ? AND transactionType=? AND (txFrom=? OR txTo=? OR nominee=?) ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                if (txType === types_1.TransactionSearchType.InternalTxReceipt) {
                    sql = `SELECT * FROM transactions WHERE cycle BETWEEN ? and ? AND (transactionType!=? AND transactionType!=? AND transactionType!=?) AND (txFrom=? OR txTo=? OR nominee=?) ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                    transactions = await db.all(sql, [
                        start,
                        end,
                        types_1.TransactionType.Receipt,
                        types_1.TransactionType.StakeReceipt,
                        types_1.TransactionType.UnstakeReceipt,
                        address,
                        address,
                        address,
                    ]);
                }
                else
                    transactions = await db.all(sql, [start, end, ty, address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                const sql = `SELECT * FROM tokenTxs WHERE cycle BETWEEN ? and ? AND (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND tokenType=? ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                transactions = await db.all(sql, [start, end, address, address, address, ty]);
            }
            else if (txType === types_1.TransactionSearchType.TokenTransfer) {
                if (filterAddress) {
                    const sql = `SELECT * FROM tokenTxs WHERE cycle BETWEEN ? and ? AND contractAddress=? AND (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND NOT (tokenType=?) ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                    transactions = await db.all(sql, [
                        start,
                        end,
                        address,
                        filterAddress,
                        filterAddress,
                        filterAddress,
                        types_1.TransactionType.EVM_Internal,
                    ]);
                }
                else {
                    const sql = `SELECT * FROM tokenTxs WHERE cycle BETWEEN ? and ? AND contractAddress=? AND NOT (tokenType=?) ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                    transactions = await db.all(sql, [start, end, address, types_1.TransactionType.EVM_Internal]);
                }
            }
        }
        else if (txType) {
            if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                // const ty = TransactionType.InternalTxReceipt
                // const sql = `SELECT * FROM transactions WHERE cycle BETWEEN ? and ? AND transactionType!=? ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`
                // transactions = await db.all(sql, [start, end, ty])
                // This seems to be faster than the above query
                const sql = `SELECT * FROM transactions WHERE cycle BETWEEN ? and ? AND (transactionType=? OR transactionType=? OR transactionType=?) ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                transactions = await db.all(sql, [
                    start,
                    end,
                    types_1.TransactionType.Receipt,
                    types_1.TransactionType.StakeReceipt,
                    types_1.TransactionType.UnstakeReceipt,
                ]);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                let sql = `SELECT * FROM transactions WHERE cycle BETWEEN ? and ? AND transactionType=? ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                if (txType === types_1.TransactionSearchType.InternalTxReceipt) {
                    sql = `SELECT * FROM transactions WHERE cycle BETWEEN ? and ? AND (transactionType!=? AND transactionType!=? AND transactionType!=?) ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                    transactions = await db.all(sql, [
                        start,
                        end,
                        types_1.TransactionType.Receipt,
                        types_1.TransactionType.StakeReceipt,
                        types_1.TransactionType.UnstakeReceipt,
                    ]);
                }
                else {
                    transactions = await db.all(sql, [start, end, ty]);
                }
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                const sql = `SELECT * FROM tokenTxs WHERE cycle BETWEEN ? and ? AND tokenType=? ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
                transactions = await db.all(sql, [start, end, ty]);
            }
        }
        else {
            const sql = `SELECT * FROM transactions WHERE cycle BETWEEN ? and ? ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
            transactions = await db.all(sql, [start, end]);
        }
        if (transactions.length > 0) {
            transactions.forEach((transaction) => {
                if ('transactionType' in transaction && transaction.transactionType)
                    deserializeDbTransaction(transaction);
                else if ('tokenType' in transaction && transaction.tokenType)
                    deserializeDbToken(transaction);
            });
        }
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('transactions betweeen cycles', transactions);
    return transactions;
}
exports.queryTransactionsBetweenCycles = queryTransactionsBetweenCycles;
async function queryTransactionCountBetweenCycles(start, end, address, txType, filterAddress) {
    let transactions = { 'COUNT(*)': 0 };
    try {
        if (address) {
            if (!txType) {
                const sql = `SELECT COUNT(*) FROM transactions WHERE cycle BETWEEN ? and ? AND (txFrom=? OR txTo=? OR nominee=?)`;
                transactions = await db.get(sql, [start, end, address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                const ty = types_1.TransactionType.InternalTxReceipt;
                const sql = `SELECT COUNT(*) FROM transactions WHERE cycle BETWEEN ? and ? AND transactionType!=? AND (txFrom=? OR txTo=? OR nominee=?)`;
                transactions = await db.get(sql, [start, end, ty, address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                let sql = `SELECT COUNT(*) FROM transactions WHERE cycle BETWEEN ? and ? AND transactionType=? AND (txFrom=? OR txTo=? OR nominee=?)`;
                if (txType === types_1.TransactionSearchType.InternalTxReceipt) {
                    sql = `SELECT COUNT(*) FROM transactions WHERE cycle BETWEEN ? and ? AND (transactionType!=? AND transactionType!=? AND transactionType!=?) AND (txFrom=? OR txTo=? OR nominee=?)`;
                    transactions = await db.get(sql, [
                        start,
                        end,
                        types_1.TransactionType.Receipt,
                        types_1.TransactionType.StakeReceipt,
                        types_1.TransactionType.UnstakeReceipt,
                        address,
                        address,
                        address,
                    ]);
                }
                else
                    transactions = await db.get(sql, [start, end, ty, address, address, address]);
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                const sql = `SELECT COUNT(*) FROM tokenTxs WHERE cycle BETWEEN ? and ? AND (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND tokenType=?`;
                transactions = await db.get(sql, [start, end, address, address, address, ty]);
            }
            else if (txType === types_1.TransactionSearchType.TokenTransfer) {
                if (filterAddress) {
                    const sql = `SELECT COUNT(*) FROM tokenTxs WHERE cycle BETWEEN ? and ? AND contractAddress=? AND (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND NOT tokenType=?`;
                    transactions = await db.get(sql, [
                        start,
                        end,
                        address,
                        filterAddress,
                        filterAddress,
                        filterAddress,
                        types_1.TransactionType.EVM_Internal,
                    ]);
                }
                else {
                    const sql = `SELECT COUNT(*) FROM tokenTxs WHERE cycle BETWEEN ? and ? AND contractAddress=? AND NOT tokenType=?`;
                    transactions = await db.get(sql, [start, end, address, types_1.TransactionType.EVM_Internal]);
                }
            }
        }
        else if (txType || txType === types_1.TransactionSearchType.All) {
            if (txType === types_1.TransactionSearchType.All) {
                const sql = `SELECT COUNT(*) FROM transactions WHERE cycle BETWEEN ? and ?`;
                transactions = await db.get(sql, [start, end]);
            }
            else if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                // const ty = TransactionType.InternalTxReceipt
                const sql = `SELECT COUNT(*) FROM transactions WHERE cycle BETWEEN ? and ? AND (transactionType=? OR transactionType=? OR transactionType=?)`;
                transactions = await db.get(sql, [
                    start,
                    end,
                    types_1.TransactionType.Receipt,
                    types_1.TransactionType.StakeReceipt,
                    types_1.TransactionType.UnstakeReceipt,
                ]);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                let sql = `SELECT COUNT(*) FROM transactions WHERE cycle BETWEEN ? and ? AND transactionType=?`;
                if (txType === types_1.TransactionSearchType.InternalTxReceipt) {
                    // This is taking too long to respond
                    // transactions = await db.get(sql, [start, end, ty])
                    sql = `SELECT COUNT(*) FROM transactions WHERE cycle BETWEEN ? and ?`;
                    transactions = await db.get(sql, [start, end]);
                    const totalTxs = transactions['COUNT(*)'] || 0;
                    sql = `SELECT COUNT(*) FROM transactions WHERE cycle BETWEEN ? and ? AND (transactionType=? OR transactionType=? OR transactionType=?)`;
                    transactions = await db.get(sql, [
                        start,
                        end,
                        types_1.TransactionType.Receipt,
                        types_1.TransactionType.StakeReceipt,
                        types_1.TransactionType.UnstakeReceipt,
                    ]);
                    const totalTxsExceptInternalTx = transactions['COUNT(*)'] || 0;
                    transactions['COUNT(*)'] = totalTxs - totalTxsExceptInternalTx;
                }
                else {
                    transactions = await db.get(sql, [start, end, ty]);
                }
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                const sql = `SELECT COUNT(*) FROM tokenTxs WHERE cycle BETWEEN ? and ? AND tokenType=?`;
                transactions = await db.get(sql, [start, end, ty]);
            }
        }
        else {
            const sql = `SELECT COUNT(*) FROM transactions WHERE cycle BETWEEN ? and ?`;
            transactions = await db.get(sql, [start, end]);
        }
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('transactions count between cycles', transactions);
    return transactions['COUNT(*)'] || 0;
}
exports.queryTransactionCountBetweenCycles = queryTransactionCountBetweenCycles;
async function queryTransactionCountByCycles(start, end, txType) {
    let transactions = [];
    try {
        if (txType === types_1.TransactionSearchType.Receipt ||
            txType === types_1.TransactionSearchType.NodeRewardReceipt ||
            txType === types_1.TransactionSearchType.StakeReceipt ||
            txType === types_1.TransactionSearchType.UnstakeReceipt ||
            txType === types_1.TransactionSearchType.InternalTxReceipt) {
            const ty = txType === types_1.TransactionSearchType.Receipt
                ? types_1.TransactionType.Receipt
                : txType === types_1.TransactionSearchType.NodeRewardReceipt
                    ? types_1.TransactionType.NodeRewardReceipt
                    : txType === types_1.TransactionSearchType.StakeReceipt
                        ? types_1.TransactionType.StakeReceipt
                        : txType === types_1.TransactionSearchType.UnstakeReceipt
                            ? types_1.TransactionType.UnstakeReceipt
                            : types_1.TransactionType.InternalTxReceipt;
            let sql = `SELECT cycle, COUNT(*) FROM transactions WHERE transactionType=? GROUP BY cycle HAVING cycle BETWEEN ? AND ? ORDER BY cycle ASC`;
            if (txType === types_1.TransactionSearchType.InternalTxReceipt) {
                sql = `SELECT cycle, COUNT(*) FROM transactions WHERE transactionType!=? AND transactionType!=? AND transactionType!=? GROUP BY cycle HAVING cycle BETWEEN ? AND ? ORDER BY cycle ASC`;
                transactions = await db.all(sql, [
                    types_1.TransactionType.Receipt,
                    types_1.TransactionType.StakeReceipt,
                    types_1.TransactionType.UnstakeReceipt,
                    start,
                    end,
                ]);
            }
            else
                transactions = await db.all(sql, [ty, start, end]);
        }
        else {
            const sql = `SELECT cycle, COUNT(*) FROM transactions GROUP BY cycle HAVING cycle BETWEEN ? AND ? ORDER BY cycle ASC`;
            transactions = await db.all(sql, [start, end]);
        }
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('Transaction count by cycles', transactions);
    return transactions.map((receipt) => {
        return {
            cycle: receipt.cycle,
            transactions: receipt['COUNT(*)'],
        };
    });
}
exports.queryTransactionCountByCycles = queryTransactionCountByCycles;
async function queryTransactionCountByTimestamp(beforeTimestamp, afterTimestamp, address, txType, filterAddress) {
    let transactions = { 'COUNT(*)': 0 };
    let sql = `SELECT COUNT(*) FROM transactions WHERE `;
    if (txType) {
        if (txType === types_1.TransactionSearchType.EVM_Internal ||
            txType === types_1.TransactionSearchType.ERC_20 ||
            txType === types_1.TransactionSearchType.ERC_721 ||
            txType === types_1.TransactionSearchType.ERC_1155 ||
            txType === types_1.TransactionSearchType.TokenTransfer)
            sql = `SELECT COUNT(*) FROM tokenTxs WHERE `;
    }
    const values = [];
    if (afterTimestamp > 0) {
        sql += `timestamp>? `;
        values.push(afterTimestamp);
    }
    if (beforeTimestamp > 0) {
        if (afterTimestamp > 0)
            sql += `AND timestamp<? `;
        else
            sql += `timestamp<? `;
        values.push(beforeTimestamp);
    }
    try {
        if (address) {
            if (!txType) {
                sql += `AND (txFrom=? OR txTo=? OR nominee=?)`;
                values.push(address, address, address);
            }
            else if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                const ty = types_1.TransactionType.InternalTxReceipt;
                sql = `AND (txFrom=? OR txTo=? OR nominee=?) AND transactionType!=?`;
                values.push(address, address, address, ty);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                sql += `AND (txFrom=? OR txTo=? OR nominee=?) AND transactionType=?`;
                values.push(address, address, address, ty);
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                sql += `AND (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND tokenType=?`;
                values.push(address, address, address, ty);
            }
            else if (txType === types_1.TransactionSearchType.TokenTransfer) {
                if (filterAddress) {
                    sql += `AND contractAddress=? AND (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND NOT tokenType=?`;
                    values.push(address, filterAddress, filterAddress, filterAddress, types_1.TransactionType.EVM_Internal);
                }
                else {
                    sql += `AND contractAddress=? AND NOT tokenType=?`;
                    values.push(address, types_1.TransactionType.EVM_Internal);
                }
            }
        }
        else if (txType) {
            if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                const ty = types_1.TransactionType.InternalTxReceipt;
                sql += `AND transactionType!=?`;
                values.push(ty);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                sql += `AND transactionType=?`;
                values.push(ty);
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                sql += `AND tokenType=?`;
                values.push(ty);
            }
        }
        transactions = await db.get(sql, values);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('transactions count by timestamp', transactions);
    return transactions['COUNT(*)'] || 0;
}
exports.queryTransactionCountByTimestamp = queryTransactionCountByTimestamp;
async function queryTransactionsByTimestamp(skip = 0, limit = 10, beforeTimestamp, afterTimestamp, address, txType, filterAddress) {
    let transactions = [];
    let sql = `SELECT * FROM transactions WHERE `;
    if (txType) {
        if (txType === types_1.TransactionSearchType.EVM_Internal ||
            txType === types_1.TransactionSearchType.ERC_20 ||
            txType === types_1.TransactionSearchType.ERC_721 ||
            txType === types_1.TransactionSearchType.ERC_1155 ||
            txType === types_1.TransactionSearchType.TokenTransfer)
            sql = `SELECT * FROM tokenTxs WHERE `;
    }
    const values = [];
    let sqlSuffix = '';
    if (afterTimestamp > 0) {
        sql += `timestamp>? `;
        sqlSuffix = ` ORDER BY timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        values.push(afterTimestamp);
    }
    if (beforeTimestamp > 0) {
        if (afterTimestamp > 0)
            sql += `AND timestamp<? `;
        else {
            sql += `timestamp<? `;
            sqlSuffix = ` ORDER BY timestamp DESC LIMIT ${limit} OFFSET ${skip}`;
        }
        values.push(beforeTimestamp);
    }
    try {
        if (address) {
            if (!txType || types_1.TransactionSearchType.All) {
                sql += `AND (txFrom=? OR txTo=? OR nominee=?)`;
                values.push(address, address, address);
            }
            else if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                const ty = types_1.TransactionType.InternalTxReceipt;
                sql += `AND (txFrom=? OR txTo=? OR nominee=?) AND transactionType!=?`;
                values.push(address, address, address, ty);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                sql += `AND (txFrom=? OR txTo=? OR nominee=?) AND transactionType=?`;
                values.push(address, address, address, ty);
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                sql += `AND (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND tokenType=?`;
                values.push(address, address, address, ty);
            }
            else if (txType === types_1.TransactionSearchType.TokenTransfer) {
                if (filterAddress) {
                    sql += `AND contractAddress=? AND (tokenFrom=? OR tokenTo=? OR tokenOperator=?) AND NOT (tokenType=?)`;
                    values.push(address, filterAddress, filterAddress, filterAddress, types_1.TransactionType.EVM_Internal);
                }
                else {
                    sql += `AND contractAddress=? AND NOT (tokenType=?)`;
                    values.push(address, types_1.TransactionType.EVM_Internal);
                }
            }
        }
        else if (txType) {
            if (txType === types_1.TransactionSearchType.AllExceptInternalTx) {
                const ty = types_1.TransactionType.InternalTxReceipt;
                sql += `AND transactionType!=?`;
                values.push(ty);
            }
            else if (txType === types_1.TransactionSearchType.Receipt ||
                txType === types_1.TransactionSearchType.NodeRewardReceipt ||
                txType === types_1.TransactionSearchType.StakeReceipt ||
                txType === types_1.TransactionSearchType.UnstakeReceipt ||
                txType === types_1.TransactionSearchType.InternalTxReceipt) {
                const ty = txType === types_1.TransactionSearchType.Receipt
                    ? types_1.TransactionType.Receipt
                    : txType === types_1.TransactionSearchType.NodeRewardReceipt
                        ? types_1.TransactionType.NodeRewardReceipt
                        : txType === types_1.TransactionSearchType.StakeReceipt
                            ? types_1.TransactionType.StakeReceipt
                            : txType === types_1.TransactionSearchType.UnstakeReceipt
                                ? types_1.TransactionType.UnstakeReceipt
                                : types_1.TransactionType.InternalTxReceipt;
                sql += `AND transactionType=?`;
                values.push(ty);
            }
            else if (txType === types_1.TransactionSearchType.EVM_Internal ||
                txType === types_1.TransactionSearchType.ERC_20 ||
                txType === types_1.TransactionSearchType.ERC_721 ||
                txType === types_1.TransactionSearchType.ERC_1155) {
                const ty = txType === types_1.TransactionSearchType.EVM_Internal
                    ? types_1.TransactionType.EVM_Internal
                    : txType === types_1.TransactionSearchType.ERC_20
                        ? types_1.TransactionType.ERC_20
                        : txType === types_1.TransactionSearchType.ERC_721
                            ? types_1.TransactionType.ERC_721
                            : types_1.TransactionType.ERC_1155;
                sql += `AND tokenType=?`;
                values.push(ty);
            }
        }
        sql += sqlSuffix;
        transactions = await db.all(sql, values);
        if (transactions.length > 0) {
            transactions.forEach((transaction) => {
                if ('transactionType' in transaction)
                    deserializeDbTransaction(transaction);
                else if ('tokenType' in transaction)
                    deserializeDbToken(transaction);
            });
        }
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('transactions by timestamp', transactions);
    return transactions;
}
exports.queryTransactionsByTimestamp = queryTransactionsByTimestamp;
// transactionCount with txType = Receipt, StakeReceipt, UnstakeReceipt
async function queryTransactionCountByBlock(blockNumber, blockHash) {
    let transactions = { 'COUNT(*)': 0 };
    let sql = `SELECT COUNT(*) FROM transactions WHERE transactionType IN (?,?,?) `;
    const values = [
        types_1.TransactionType.Receipt,
        types_1.TransactionType.StakeReceipt,
        types_1.TransactionType.UnstakeReceipt,
    ];
    if (blockNumber > 0) {
        sql += `AND blockNumber=? `;
        values.push(blockNumber);
    }
    else if (blockHash) {
        sql += `AND blockHash=? `;
        values.push(blockHash);
    }
    try {
        transactions = await db.get(sql, values);
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('transactions count by block', transactions);
    return transactions['COUNT(*)'] || 0;
}
exports.queryTransactionCountByBlock = queryTransactionCountByBlock;
// transactions with txType = Receipt, StakeReceipt, UnstakeReceipt
async function queryTransactionsByBlock(blockNumber, blockHash) {
    let transactions = [];
    let sql = `SELECT * FROM transactions WHERE transactionType IN (?,?,?) `;
    const values = [
        types_1.TransactionType.Receipt,
        types_1.TransactionType.StakeReceipt,
        types_1.TransactionType.UnstakeReceipt,
    ];
    if (blockNumber >= 0) {
        sql += `AND blockNumber=? `;
        values.push(blockNumber);
    }
    else if (blockHash) {
        sql += `AND blockHash=? `;
        values.push(blockHash);
    }
    sql += `ORDER BY timestamp ASC;`;
    try {
        transactions = await db.all(sql, values);
        if (transactions.length > 0) {
            transactions.forEach((transaction) => deserializeDbTransaction(transaction));
        }
    }
    catch (e) {
        console.log(e);
    }
    if (index_1.config.verbose)
        console.log('transactions by block', transactions);
    return transactions;
}
exports.queryTransactionsByBlock = queryTransactionsByBlock;
async function queryTokenTxByTxId(txId) {
    try {
        const sql = `SELECT * FROM tokenTxs WHERE txId=?`;
        const tokenTxs = await db.all(sql, [txId]);
        if (index_1.config.verbose)
            console.log('tokenTxs txId', tokenTxs);
        return tokenTxs;
    }
    catch (e) {
        console.log(e);
    }
    return [];
}
exports.queryTokenTxByTxId = queryTokenTxByTxId;
function deserializeDbTransaction(transaction) {
    transaction.wrappedEVMAccount = lib_types_1.Utils.safeJsonParse(transaction.wrappedEVMAccount);
    transaction.originalTxData = lib_types_1.Utils.safeJsonParse(transaction.originalTxData);
}
function deserializeDbToken(transaction) {
    transaction.contractInfo = lib_types_1.Utils.safeJsonParse(transaction.contractInfo);
}
//# sourceMappingURL=transaction.js.map