import { WrappedEVMAccount, Token } from './account';
export interface Transaction {
    txId: string;
    cycle: number;
    blockNumber: number;
    blockHash: string;
    timestamp: number;
    wrappedEVMAccount: WrappedEVMAccount;
    transactionType: TransactionType;
    txHash: string;
    txFrom: string;
    txTo: string;
    nominee?: string;
    originalTxData: unknown;
    tokenTxs?: TokenTx[];
    txStatus?: TxStatus;
    data?: unknown;
}
export type TxStatus = 'Pending' | 'Expired';
export declare enum TransactionType {
    Receipt = 0,
    NodeRewardReceipt = 1,
    StakeReceipt = 2,
    UnstakeReceipt = 3,
    EVM_Internal = 4,
    ERC_20 = 5,
    ERC_721 = 6,
    ERC_1155 = 7,
    InternalTxReceipt = 8
}
export interface ContractInfo {
    totalSupply: number;
    decimals: string;
    name: string;
    symbol: string;
}
export interface TokenTx {
    cycle: number;
    timestamp: number;
    contractAddress: string;
    contractInfo?: ContractInfo;
    tokenFrom: string;
    tokenTo: string;
    tokenValue: string;
    tokenType: TransactionType;
    tokenEvent: string;
    tokenOperator?: string | null;
    transactionFee: string;
    txId?: string;
    txHash: string;
}
export declare enum InternalTXType {
    SetGlobalCodeBytes = 0,
    InitNetwork = 1,
    NodeReward = 2,
    ChangeConfig = 3,
    ApplyChangeConfig = 4,
    SetCertTime = 5,
    Stake = 6,
    Unstake = 7,
    InitRewardTimes = 8,
    ClaimReward = 9,
    ChangeNetworkParam = 10,
    ApplyNetworkParam = 11,
    Penalty = 12
}
/**
 * InternalTx is a non EVM TX that shardeum can use for utility task such as global changes
 *
 */
export interface InternalTxBase {
    isInternalTx: boolean;
    internalTXType: InternalTXType;
}
export interface InternalTx extends InternalTxBase {
    timestamp: number;
    from?: string;
    to?: string;
    accountData?: WrappedEVMAccount;
    network?: string;
    nodeId?: string;
}
export interface DecodeTxResult {
    txs: TokenTx[];
    accs: string[];
    tokens: Token[];
}
export declare enum TransactionSearchType {
    All = 0,
    Receipt = 1,
    NodeRewardReceipt = 2,
    StakeReceipt = 3,
    UnstakeReceipt = 4,
    EVM_Internal = 5,
    ERC_20 = 6,
    ERC_721 = 7,
    ERC_1155 = 8,
    TokenTransfer = 9,
    InternalTxReceipt = 10,
    AllExceptInternalTx = 11,
    Pending = 12
}
export interface Log {
    address: string;
    blockHash: string;
    blockNumber: string;
    data: string;
    logIndex: string;
    topics: string[];
    transactionHash: string;
    transactionIndex: string;
}
