"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeEVMRawTxData = exports.getStakeTxBlobFromEVMTx = exports.isStakingEVMTx = exports.getTransactionObj = void 0;
const tx_1 = require("@ethereumjs/tx");
const util_1 = require("@ethereumjs/util");
const config_1 = require("../config");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const types_1 = require("../types");
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function getTransactionObj(tx) {
    if (!tx.raw)
        throw Error('tx has no raw field');
    let transactionObj;
    const serializedInput = (0, util_1.toBytes)(tx.raw);
    try {
        transactionObj = tx_1.TransactionFactory.fromSerializedData(serializedInput);
    }
    catch (e) {
        /* prettier-ignore */ if (config_1.config.verbose)
            console.log('Unable to get legacy transaction obj', e);
    }
    if (!transactionObj) {
        try {
            transactionObj =
                tx_1.TransactionFactory.fromSerializedData(serializedInput);
        }
        catch (e) {
            /* prettier-ignore */ if (config_1.config.verbose)
                console.log('Unable to get transaction obj', e);
        }
    }
    if (transactionObj) {
        return transactionObj;
    }
    else {
        throw Error('tx obj fail');
    }
}
exports.getTransactionObj = getTransactionObj;
const stakeTargetAddress = '0x0000000000000000000000000000000000000001'; //dev-relaunch required to change this '0x0000000000000000000000000000000000010000',
function isStakingEVMTx(transaction) {
    if (transaction.to && transaction.to.toString() === stakeTargetAddress)
        return true;
    return false;
}
exports.isStakingEVMTx = isStakingEVMTx;
function getStakeTxBlobFromEVMTx(transaction) {
    try {
        const stakeTxString = (0, util_1.toAscii)((0, util_1.bytesToHex)(transaction.data));
        return lib_types_1.Utils.safeJsonParse(stakeTxString);
    }
    catch (e) {
        console.log('Unable to get stakeTxBlobFromEVMTx', e);
    }
}
exports.getStakeTxBlobFromEVMTx = getStakeTxBlobFromEVMTx;
function decodeEVMRawTxData(originalTxData) {
    if (originalTxData.originalTxData.tx.raw) {
        // EVM Tx
        const txObj = getTransactionObj(originalTxData.originalTxData.tx);
        // Custom readableReceipt for originalTxsData
        if (txObj) {
            const readableReceipt = {
                from: txObj.getSenderAddress().toString(),
                to: txObj.to ? txObj.to.toString() : null,
                nonce: txObj.nonce.toString(16),
                value: txObj.value.toString(16),
                data: '0x' + txObj.data.toString(),
            };
            if (originalTxData.transactionType === types_1.TransactionType.StakeReceipt ||
                originalTxData.transactionType === types_1.TransactionType.UnstakeReceipt) {
                const internalTxData = getStakeTxBlobFromEVMTx(txObj);
                readableReceipt['internalTxData'] = internalTxData;
            }
            originalTxData.originalTxData = { ...originalTxData.originalTxData, readableReceipt };
        }
    }
}
exports.decodeEVMRawTxData = decodeEVMRawTxData;
//# sourceMappingURL=decodeEVMRawTx.js.map